using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Entities.Modules.Definitions;
using DotNetNuke.Entities.Portals;
using DotNetNuke.Security;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Services.Localization;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;
using System.Xml;

namespace DotNetNuke.Modules.NBrightStore
{

    /// -----------------------------------------------------------------------------
    /// <summary>
    /// The EditNBrightIndex class is used to manage content
    /// </summary>
    /// -----------------------------------------------------------------------------
    public partial class AdminCategories : NBrightStoreAdminBase 
    {

        #region Load Event Handlers

        private string _dataTemplate;

        override protected void OnInit(EventArgs e)
        {

            //Set base EntityTypeCode, MUST be done for all NBright child ctrls before OnInit
            base.EntityTypeCode = "CATEGORY";
            base.EntityTypeCodeLang = "CATEGORYLANG";
            base.CtrlTypeCode = "AdminCategories";

            base.UploadFolder = "NBrightStoreImages";

			//Remove any cache for the module -1, we don't want any cache in BO
			//All category records are portal wide, hence -1 in cahce key.
			NBrightStoreUtils.RemoveModCache(-1);

            base.OnInit(e);

            // Installation wide template, so use controlpath, so templates are stored in the module file structure.
            if (base.ItemId == "")
            {
                rpData.Visible = false;
                //All list display functions are processed via the parent class, we just initiallize here.
                base.OnInitActivateList(true,true,MapPath(ControlPath));
            }
            else
            {
                _dataTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_Detail.html", Utils.GetCurrentCulture());
                rpData.ItemTemplate = new GenXmlTemplate(_dataTemplate);

				base.LangaugeTemplate = TemplCtrl.GetTemplateData(EntityTypeCodeLang + "_Detail.html", Utils.GetCurrentCulture());
                base.ActivateLangaugeTemplate();
                rpLang.ItemTemplate = new GenXmlTemplate(base.LangaugeTemplate);
            }

        }

        protected override void OnLoad(EventArgs e)
        {
            try
            {
                // over ride the normal user display list with the category structure                
                base.OverRideInfoList = CategoryUtils.BuildCatList(UInfo.SearchFilters);

                base.OnLoad(e);
            }
            catch (Exception)
            {
                UInfo.ClearSearchData();
                throw;
            }
        }

        #endregion

        #region "Event handlers"

        public override void EventListItemCommand(object source, RepeaterCommandEventArgs e)
        {
            var cArg = e.CommandArgument.ToString();

            switch (e.CommandName.ToLower())
            {
                case "viewentries":
                    Response.Redirect(EditUrl("catid", cArg, "Edit"));
                    break;
				case "delete":
					if (Utils.IsNumeric(cArg) && (Convert.ToInt32(cArg) > 0))
					{
						base.DeleteData(Convert.ToInt32(cArg), UploadFolder);
					}
					Response.Redirect(EditUrl("Categories"));
					break;
			}
        }


        #endregion

        #region "Methods"

        protected override void EventAfterUpdateData(NBrightInfo objInfo)
        {
			//Make sure the records are portalwide by making the moduleid = -1
			#region "code section"
			objInfo.ModuleId = -1;
			var l = base.GetList("CATEGORYLANG", "", " and moduleid = " + ModuleId + " ");
			foreach (var i in l)
			{
				i.ModuleId = -1;
				ModCtrl.UpdateInfo(i);
			}
			#endregion

            ////update default category. Can only be 1
            //#region "code section"
            //var defcatid = NBSettings.GetXmlProperty("genxml/hidden/defaultcatid");
            //if (defcatid != base.ItemId & objInfo.GetXmlProperty("genxml/checkbox/chkdefaultcat").ToLower() == "true")
            //{
            //    // remove checkbox on current default cat
            //    if (Utils.IsNumeric(defcatid))
            //    {
            //        var obj = ModCtrl.GetInfo(Convert.ToInt32(defcatid));
            //        obj.SetXmlProperty("genxml/checkbox/chkdefaultcat", "False");
            //        ModCtrl.UpdateInfo(obj);
            //    }
            //    // set the default category
            //    NBSettings.SetXmlProperty("genxml/hidden/defaultcatid", base.ItemId, TypeCode.String, false);
            //    ModCtrl.UpdateInfo(NBSettings);
            //}

            //if (defcatid == base.ItemId & objInfo.GetXmlProperty("genxml/checkbox/chkdefaultcat").ToLower() == "false")
            //{
            //    // No default category anymore, so remove the default category in settings
            //    NBSettings.SetXmlProperty("genxml/hidden/defaultcatid", "", TypeCode.String, false);
            //    ModCtrl.UpdateInfo(NBSettings);
            //}
            //#endregion

			// update if this is a sub category, by getting the parentid, set to parentid to "0" to make it root node again. 
			#region "code section"
            if (Utils.IsNumeric(objInfo.GetXmlProperty("genxml/dropdownlist/ddlparentcatid")))
            {
                objInfo.ParentItemId = Convert.ToInt32(objInfo.GetXmlProperty("genxml/dropdownlist/ddlparentcatid"));
                if (objInfo.ParentItemId != objInfo.ItemID)
                {
                    ModCtrl.UpdateInfo(objInfo);
                }
                else
                {
                    objInfo.ParentItemId = 0; // cannot be same as itemid, set to root to stop ifinate loop.
                    objInfo.SetXmlProperty("genxml/dropdownlist/ddlparentcatid", "0");
                    ModCtrl.UpdateInfo(objInfo);
                }
            }
            else
            {
                if (objInfo.ParentItemId != 0)
                {
                    objInfo.ParentItemId = 0; // cannot be same as itemid, set to root to stop ifinate loop.
                    objInfo.SetXmlProperty("genxml/dropdownlist/ddlparentcatid", "0");
                    ModCtrl.UpdateInfo(objInfo);
                }
            }
			#endregion

			//write a recordsortorder, if it doesn;t exists.
			#region "code section"
			if (!Utils.IsNumeric(objInfo.GetXmlProperty("genxml/hidden/recordsortorder")))
            {
                var c = ModCtrl.GetListInfoCount(PortalId, ModuleId, EntityTypeCode);
                objInfo.SetXmlProperty("genxml/hidden/recordsortorder", c.ToString("0000000"));
			}
			#endregion

			//Update the parent catid list, so cascading results can work.
            objInfo.SetXmlProperty("genxml/hidden/parentcatids", CategoryUtils.GetParentCatIds(objInfo.ItemID));
            
			ModCtrl.UpdateInfo(objInfo);

        }



        #endregion

    }

}