using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Entities.Modules.Actions;
using DotNetNuke.Security;
using DotNetNuke.Services.Localization;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;
using System.Xml;

namespace DotNetNuke.Modules.NBrightStore
{
	public partial class AdminOptions : NBrightStoreAdminBase
	{

		#region Load Event Handlers

        private String _EditAction = ""; 

        override protected void OnInit(EventArgs e)
        {

			//Set base EntityTypeCode, MUST be done for all NBright child ctrls before OnInit
			base.EntityTypeCode = "OPT";
            base.EntityTypeCodeLang = "OPTLANG";
			base.CtrlTypeCode = "AdminOptions";

			base.OnInit(e);

			// Installation wide template, so use controlpath, so templates are stored in the module file structure.
			if (base.ItemId == "")
			{
				//All list display functions are processed via the parent class, we just initialize here.
				base.OnInitActivateList(true, true, MapPath(ControlPath));

			}
			else
			{
				// All data templates process by parent class, we just initialize here.
				base.OnInitActivateData();
			}

            //Apply template to rpoptionval
            var strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ValH.html", Utils.GetCurrentCulture());
            rpOptionValH.ItemTemplate = new GenXmlTemplate(strTemplate);
            strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_Val.html", Utils.GetCurrentCulture());
            rpOptionVal.ItemTemplate = new GenXmlTemplate(strTemplate);
            strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ValLang.html", Utils.GetCurrentCulture());
            rpOptionValLang.ItemTemplate = new GenXmlTemplate(strTemplate);
            strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ValF.html", Utils.GetCurrentCulture());
            rpOptionValF.ItemTemplate = new GenXmlTemplate(strTemplate);


		}

		#endregion

        protected void CtrlOptionItemCommand(object source, RepeaterCommandEventArgs e)
        {
            var obj = base.GetData();
            var num = 0;
            switch (e.CommandName.ToLower())
            {
                case "addoptionval":
                    num = GenXmlFunctions.GetFieldAsInteger((Repeater)source, "txtOptionValAmount");
                    if (num == 0) num = 1;
                    obj = ProductFunctions.AddEntity(obj, "optionval", num);
                    ModCtrl.UpdateInfo(obj);
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode));
                    break;

            }

        }

        protected override void EventCtrlEditItemCommand(object source, RepeaterCommandEventArgs e)
        {
            _EditAction = e.CommandName.ToLower();
        }

		protected override void EventAfterUpdateData(NBrightInfo objInfo)
		{

            #region "Entities"

            if (objInfo != null)
            {
                String ent = "optionval";
                var rpEntity = new Repeater();
                var rpEntityLang = new Repeater();
                var folderMapPath = "";

                foreach (String e in ent.Split(','))
                {

                    switch (e)
                    {
                        case "optionval":
                            rpEntity = rpOptionVal;
                            rpEntityLang = rpOptionValLang;
                            break;
                    }

                    objInfo = ProductFunctions.InsertEntityData(objInfo, rpEntity, e, folderMapPath);

                    ModCtrl.UpdateInfo(objInfo);

                    var objLangInfo = GetData(objInfo.ItemID.ToString(""), base.EntityLangauge);
                    objLangInfo = ProductFunctions.InsertEntityLangData(objLangInfo, rpEntity, rpEntityLang, e);
                    ModCtrl.UpdateInfo(objLangInfo);
                }

                if (UInfo.FromItemId != "" && _EditAction == "updateexit") // only want this to trigger when we've finished.
                {
                    // deal with product level images
                    var objPInfo = ModCtrl.GetInfo(Convert.ToInt32(UInfo.FromItemId));
                    if (objPInfo != null)
                    {
                        objInfo.AddXref("prdopt", UInfo.FromItemId);
                        objPInfo.AddXref("prdopt", objInfo.ItemID.ToString());

                        //calculate models 
                        objPInfo = ProductFunctions.CalculateModels(objPInfo, Server.MapPath("/DesktopModules/NBright/NBrightStore"));

                        ModCtrl.UpdateInfo(objInfo);
                        ModCtrl.UpdateInfo(objPInfo);
                        if (UInfo.RtnUrl != "")
                        {
                            var rtnUrl = UInfo.RtnUrl;

                            //clear down user redirect data
                            UInfo.FromItemId = "";
                            UInfo.RtnUrl = "";
                            UInfo.Save();

                            Response.Redirect(rtnUrl);
                        }
                    }
                }
            }

            #endregion



		}

        protected override void EventAfterLoadDisplayData()
        {

            var objInfo = GetData();
            if (objInfo != null)
            {

                var objlangInfo = GetData(base.EntityLangauge);

                #region "Models"

                base.DoDisplay(rpOptionValH);

                var l = ProductFunctions.GetEntityList(objInfo, "optionval");
                if (l.Count == 0)
                {
                    //no Model Data, must be new product so add single empty model
                    base.DoDisplay(rpOptionVal);
                    base.DoDisplay(rpOptionValLang);
                }
                else
                {
                    rpOptionVal.DataSource = l;
                    rpOptionVal.DataBind();
                    if (objlangInfo != null)
                    {
                        l = ProductFunctions.GetEntityLangList(objInfo, objlangInfo, "optionval");
                        rpOptionValLang.DataSource = l;
                        rpOptionValLang.DataBind();
                    }

                }

                base.DoDisplay(rpOptionValF);

                #endregion

            }
        }

	}
}
