using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Entities.Modules.Actions;
using DotNetNuke.Security;
using DotNetNuke.Services.Localization;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;
using System.Xml;

namespace DotNetNuke.Modules.NBrightStore
{
	public partial class AdminProducts : NBrightStoreAdminBase
	{
        string _tempXml = "";

		#region Load Event Handlers

        override protected void OnInit(EventArgs e)
        {

			//Set base EntityTypeCode, MUST be done for all NBright child ctrls before OnInit
			base.EntityTypeCode = "PRD";
			base.EntityTypeCodeLang = "PRDLANG";
			base.CtrlTypeCode = "AdminProducts";

			base.OnInit(e);

			// Installation wide template, so use controlpath, so templates are stored in the module file structure.
			if (base.ItemId == "")
			{
				//All list display functions are processed via the parent class, we just initialize here.
				base.OnInitActivateList(true, true, MapPath(ControlPath));

			}
			else
			{
				// All data templates process by parent class, we just initialize here.
				base.OnInitActivateData();
                
                //show categories products
                Literal DisplayCat = new Literal();
                DisplayCat.Text = ProductFunctions.GetRelatedCats(PortalId, base.ItemId, NBrightDNN.DnnUtils.GetCurrentValidCultureCode(), "AdminCatXref", Server.MapPath(base.ControlPath));
                phCategories.Controls.Add(DisplayCat);

                //show releated products
                Literal DisplayRelPrd = new Literal();
                DisplayRelPrd.Text = ProductFunctions.GetRelatedProducts(PortalId, base.ItemId, NBrightDNN.DnnUtils.GetCurrentValidCultureCode(), "AdminProductXref", Server.MapPath(base.ControlPath));
                phAttributes.Controls.Add(DisplayRelPrd);

                //show product images
                Literal DisplayPrdImg = new Literal();
                DisplayPrdImg.Text = ProductFunctions.GetProductImgs(PortalId, base.ItemId, NBrightDNN.DnnUtils.GetCurrentValidCultureCode(), "AdminProductImg", Server.MapPath(base.ControlPath));
                phAttributes.Controls.Add(DisplayPrdImg);

                //show product images
                Literal DisplayPrdOpt = new Literal();
                DisplayPrdOpt.Text = ProductFunctions.GetProductOpts(PortalId, base.ItemId, NBrightDNN.DnnUtils.GetCurrentValidCultureCode(), "AdminProductOpt", Server.MapPath(base.ControlPath));
                phAttributes.Controls.Add(DisplayPrdOpt);

			}


			//Apply template to rpModels
			var strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ModelsH.html", Utils.GetCurrentCulture());
			rpModelsH.ItemTemplate = new GenXmlTemplate(strTemplate);
			strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_Models.html", Utils.GetCurrentCulture());
			rpModels.ItemTemplate = new GenXmlTemplate(strTemplate);
			strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ModelsLang.html", Utils.GetCurrentCulture());
			rpModelsLang.ItemTemplate = new GenXmlTemplate(strTemplate);
			strTemplate = TemplCtrl.GetTemplateData(CtrlTypeCode + "_ModelsF.html", Utils.GetCurrentCulture());
			rpModelsF.ItemTemplate = new GenXmlTemplate(strTemplate);

		}

		#endregion

		#region "Extra data Events"

        protected void CtrlProductItemCommand(object source, RepeaterCommandEventArgs e)
        {
            var obj = base.GetData();
            var num = 0;
            switch (e.CommandName.ToLower())
            {
                case "addmodel":
                    num = GenXmlFunctions.GetFieldAsInteger((Repeater)source, "txtModelAmount");
                    if (num == 0) num = 1;
                    obj = ProductFunctions.AddEntity(obj,"models",num);
                    ModCtrl.UpdateInfo(obj);
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode));
                    break;
            }

        }

        protected override void EventBeforeUpdateData()
        {
            var objInfo = GetData();
            if (objInfo != null)
            {
                _tempXml = "<root>";
                _tempXml += "<prdxref>" + objInfo.GetXmlNode("genxml/prdxref") + "</prdxref>";
                _tempXml += "<prdimg>" + objInfo.GetXmlNode("genxml/prdimg") + "</prdimg>";
                _tempXml += "<prdopt>" + objInfo.GetXmlNode("genxml/prdopt") + "</prdopt>";
                _tempXml += "</root>";
            }
        }

		protected override void EventAfterUpdateData(NBrightInfo objInfo)
		{


			#region "Entities"

            if (objInfo != null)
            {

                if (_tempXml != "") objInfo.AddXmlNode(_tempXml, "root/prdxref", "genxml");
                if (_tempXml != "") objInfo.AddXmlNode(_tempXml, "root/prdimg", "genxml");
                if (_tempXml != "") objInfo.AddXmlNode(_tempXml, "root/prdopt", "genxml");

                String ent = "models";
                var rpEntity = new Repeater();
                var rpEntityLang = new Repeater();
            	var folderMapPath = "";

                foreach (String e in ent.Split(','))
                {

                    switch (e)
                    {
                        case "models":
                            rpEntity = rpModels;
                            rpEntityLang = rpModelsLang;
                            break;
                    }

					objInfo = ProductFunctions.InsertEntityData(objInfo, rpEntity, e, folderMapPath);

                    ModCtrl.UpdateInfo(objInfo);

                    var objLangInfo = GetData(objInfo.ItemID.ToString(""), base.EntityLangauge);
                    objLangInfo = ProductFunctions.InsertEntityLangData(objLangInfo, rpEntity, rpEntityLang, e);
                    ModCtrl.UpdateInfo(objLangInfo);
                }
            }

		    #endregion


		}

        protected override void EventAfterLoadDisplayData()
        {

            var objInfo = GetData();
            if (objInfo != null)
            {

                var objlangInfo = GetData(base.EntityLangauge);

                #region "Models"

                base.DoDisplay(rpModelsH);

                var l = ProductFunctions.GetEntityList(objInfo, "models");
                if (l.Count == 0)
                {
                    //no Model Data, must be new product so add single empty model
                    base.DoDisplay(rpModels);
                    base.DoDisplay(rpModelsLang);
                }
                else
                {
                    rpModels.DataSource = l;
                    rpModels.DataBind();
                    if (objlangInfo != null)
                    {
                        l = ProductFunctions.GetEntityLangList(objInfo, objlangInfo, "models");
                        rpModelsLang.DataSource = l;
                        rpModelsLang.DataBind();
                    }

                }

                base.DoDisplay(rpModelsF);

                #endregion

                #region "Options"

                #endregion

                #region "Documents"

                #endregion

            }
        }

	    #endregion

	}
}
