﻿using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using DotNetNuke.Common;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Modules.NBrightStore.Components;
using DotNetNuke.Services.Exceptions;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;

namespace DotNetNuke.Modules.NBrightStore
{

	public class NBrightStoreAdminBase : NBrightDNN.controls.BaseAdminPage
	{

		public const string SkinSrcAdmin = "?SkinSrc=%2fDesktopModules%2fNBright%2fNBrightStore%2fSkins%2fNWB_nbright%2fNormal";

		public NBrightInfo NBSettings;
		public NBrightStoreController ModCtrl;
		public bool DebugMode = false;
		public string UploadFolder = "";
		public string SelUserId = "";
		public string LangaugeTemplate = "";

		protected RadioButtonList rblSelectLang;
		protected Repeater rpDataH;
		protected Repeater rpData;
		protected Repeater rpLang;
		protected Repeater rpDataF;

		protected override void OnInit(EventArgs e)
		{
            // set the base url for the backoffice page
            if (NBrightStoreUtils.IsPageBasedBackOffice(Request))
            {
                OverrideEditUrlBase = NBrightStoreUtils.GetManagerPageBaseUrl();
            }

			//Remove any cache for the module, we don't want any cache in/after the BO editing.
			NBrightStoreUtils.RemoveModCache(ModuleId);

			// add required controls.
			rblSelectLang = new RadioButtonList();
			this.Controls.Add(rblSelectLang);

			rblSelectLang.SelectedIndexChanged += new EventHandler(SelectedIndexChange);
			rblSelectLang.CssClass = "rblSelectLang";
			rblSelectLang.AutoPostBack = true;
			rblSelectLang.RepeatColumns = 8;
			rblSelectLang.RepeatDirection = RepeatDirection.Horizontal;
			rblSelectLang.CausesValidation = true;

			// add required controls.
			rpDataH = new Repeater();
			this.Controls.Add(rpDataH);
			rpLang = new Repeater();
			this.Controls.Add(rpLang);
			rpData = new Repeater();
			this.Controls.Add(rpData);
			rpDataF = new Repeater();
			this.Controls.Add(rpDataF);

			rpDataH.ItemCommand += new RepeaterCommandEventHandler(CtrlEditItemCommand);
			rpData.ItemCommand += new RepeaterCommandEventHandler(CtrlEditItemCommand);
			rpLang.ItemCommand += new RepeaterCommandEventHandler(CtrlEditItemCommand);
			rpDataF.ItemCommand += new RepeaterCommandEventHandler(CtrlEditItemCommand);

			//set default Data controller
			var obj = new NBrightStoreController();
			base.ObjCtrl = obj;

			ModCtrl = (NBrightStoreController)base.ObjCtrl;

			//add template provider to NBright Templating
            NBrightCore.providers.GenXProviderManager.AddProvider("NBrightStore,DotNetNuke.Modules.NBrightStore.Components.GenXmlTemplateExt");
			var pInfo = ModCtrl.GetInfoByGuidKey(PortalId, -1, "PROVIDERS", "NBrightTempalteProviders");
			if (pInfo != null)
			{
				NBrightCore.providers.GenXProviderManager.AddProvider(pInfo.XMLDoc);
			}

			base.OnInit(e);

		}


		#region "Page Load"


		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			if (Page.IsPostBack == false)
			{
				PageLoad();
			}
		}

		private void PageLoad()
		{
			try
			{

				if ((base.ItemId == ""))
				{

					EventBeforeLoadBindListData();

					// display list   
					base.BindListData();

					EventBeforeLoadBindListData();
				}
				else
				{
					EventBeforeLoadDisplayData();

					//do header
					base.DoDisplay(rpDataH);
					// display the detail
					base.DoDetailWithLang(rpData, base.EntityLangauge, SelUserId);

					if (rblSelectLang.Items.FindByValue(base.EntityLangauge) == null)
					{
						rblSelectLang.SelectedIndex = 0;
						base.EntityLangauge = rblSelectLang.SelectedValue;
					}
					if (rblSelectLang.Items.FindByValue(base.EntityLangauge) != null)
					{
						rblSelectLang.SelectedValue = base.EntityLangauge;
						base.DoDetail(rpLang, base.EntityLangauge, ModuleId, SelUserId);
					}
					//do footer
					base.DoDisplay(rpDataF);

					EventAfterLoadDisplayData();
				}

			}
			catch (Exception exc) //Module failed to load
			{
				UInfo.ClearSearchData();
				Exceptions.ProcessModuleLoadException(this, exc);
			}
		}

		protected virtual void EventBeforeLoadDisplayData()
		{

		}

		protected virtual void EventAfterLoadDisplayData()
		{

		}

		protected virtual void EventBeforeLoadBindListData()
		{

		}

		protected virtual void EventAfterLoadBindListData()
		{

		}


		#endregion

		public void ActivateLangaugeTemplate()
		{
			if (LangaugeTemplate == "")
			{// Hide langauges if no langauge template
				rblSelectLang.Visible = false;
				base.EntityLangauge = Utils.GetCurrentCulture();
			}
			else
			{
				// Multi-Langauge template must have "lang" hidden field, so check it's there
				if (!LangaugeTemplate.ToLower().Contains("id=\"lang\""))
				{
					LangaugeTemplate = "[<tag id=\"lang\" type=\"hidden\" value=\"\" />]" + LangaugeTemplate;
				}

				PopulateSelectLang();
			}

		}

		public void OnInitActivateData()
		{
			OnInitActivateData(CtrlTypeCode + "_DataH.html", CtrlTypeCode + "_Data.html", CtrlTypeCode + "_DataLang.html", CtrlTypeCode + "_DataF.html");
		}

		public void OnInitActivateData(String headerTemplateName, String dataTemplateName, String langTemplateName, String footerTemplateName)
		{
			var strTemplate = TemplCtrl.GetTemplateData(headerTemplateName, Utils.GetCurrentCulture());
			if (!strTemplate.StartsWith("NO TEMPLATE")) rpDataH.ItemTemplate = new GenXmlTemplate(strTemplate);

			strTemplate = TemplCtrl.GetTemplateData(footerTemplateName, Utils.GetCurrentCulture());
			if (!strTemplate.StartsWith("NO TEMPLATE")) rpDataF.ItemTemplate = new GenXmlTemplate(strTemplate);

			strTemplate = TemplCtrl.GetTemplateData(dataTemplateName, Utils.GetCurrentCulture());
			if (!strTemplate.StartsWith("NO TEMPLATE")) rpData.ItemTemplate = new GenXmlTemplate(strTemplate);

			//use base.LangaugeTemplate so we can activate the language template, this var is used as a flag to allow activation
			LangaugeTemplate = TemplCtrl.GetTemplateData(langTemplateName, Utils.GetCurrentCulture());
			if (LangaugeTemplate.StartsWith("NO TEMPLATE"))
			{
				LangaugeTemplate = "";
			}
			else
			{
				rpLang.ItemTemplate = new GenXmlTemplate(LangaugeTemplate);
			}

			ActivateLangaugeTemplate();
		}


		protected void SelectedIndexChange(Object sender, EventArgs e)
		{
			if (Page.IsValid)
			{
				EventBeforeSelectLang();
				base.EntityLangauge = rblSelectLang.SelectedValue;
				var param = new string[1];
				param[0] = "lang=" + rblSelectLang.SelectedValue;
				Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param));
			}
		}

		private void PopulateSelectLang()
		{
			var vLang = new List<string>();
			var objValidLangs = ModCtrl.GetInfoByGuidKey(PortalId, -1, "DATA", "validlanguages", SelUserId);
			if (objValidLangs != null)
			{
				var nodList = objValidLangs.XMLDoc.SelectNodes("genxml/checkboxlist/chkvalidlanguages/chk");
				if (nodList != null)
				{
					foreach (XmlNode nod in nodList)
					{
						if (nod.Attributes != null && nod.Attributes["value"].InnerText.ToLower() == "false")
						{
							vLang.Add(nod.Attributes["data"].InnerText);
						}
					}
				}
			}
			rblSelectLang.Items.Clear();
			foreach (var l in NBrightDNN.DnnUtils.GetCultureCodeList())
			{
				if (!vLang.Contains(l))
				{
					var li = new ListItem { Value = l, Text = "<img src='" + ControlAdminPath.TrimEnd('/') + "/ui/images/flags/32/" + l + ".png' title='" + l + "'/>" };
					rblSelectLang.Items.Add(li);
				}
			}
		}


		protected void CtrlEditItemCommand(object source, RepeaterCommandEventArgs e)
		{
			EventCtrlEditItemCommand(source, e);
			var cArg = e.CommandArgument.ToString();
			var param = new List<string>();

			if (base.EntityLangauge != "")
			{
				param.Add("lang=" + base.EntityLangauge);
			}

			switch (e.CommandName.ToLower())
			{
				case "new":
					UInfo.ClearSearchData(); // clear search data when new add, to remove any possible invalid search filters.
					Response.Redirect(EditUrl("itemid", "0", base.CtrlTypeCode), true);
					break;
				case "update":
					UpdateData();
					Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param.ToArray()));
					break;
				case "updateexit":
					UpdateData();
					Response.Redirect(EditUrl(base.CtrlTypeCode));
					break;
				case "cancel":
					Response.Redirect(EditUrl(base.CtrlTypeCode));
					break;
				case "deletelang":
					if (Utils.IsNumeric(base.ItemId) && (Convert.ToInt32(base.ItemId) > 0))
					{
						var o = ModCtrl.GetInfo(Convert.ToInt32(base.ItemIdLang));
						if (o != null)
						{
							base.DeleteData(Convert.ToInt32(base.ItemIdLang), UploadFolder);
						}
					}
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param.ToArray()));
					break;
				case "delete":
					if (Utils.IsNumeric(base.ItemId) && (Convert.ToInt32(base.ItemId) > 0))
					{
						base.DeleteData(Convert.ToInt32(base.ItemId), UploadFolder);
					}
					Response.Redirect(EditUrl(base.CtrlTypeCode));
					break;
				case "deletefile":
					UpdateData();
					var objInfo = base.GetData();
					if (objInfo != null)
					{
						objInfo.XMLData = GenXmlFunctions.DeleteFileByCtrlName(objInfo.XMLData, cArg, PortalSettings.HomeDirectoryMapPath + UploadFolder);
						base.UpdateData(objInfo);
					}
					objInfo = base.GetDataLang();
					if (objInfo != null)
					{
						objInfo.XMLData = GenXmlFunctions.DeleteFileByCtrlName(objInfo.XMLData, cArg, PortalSettings.HomeDirectoryMapPath + UploadFolder);
						base.UpdateData(objInfo);
					}
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param.ToArray()));
					break;
				case "uploadfile":
					UpdateData();
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param.ToArray()));
					break;
				case "forcedownload":
					GenXmlFunctions.ForceDocDownload((Repeater)source, cArg, PortalSettings.HomeDirectoryMapPath + UploadFolder, Response);
					break;
				case "copyfrom":
					CopyDataLang(GenXmlFunctions.GetField((Repeater)source, "ddlcopylang"), base.EntityLangauge, UploadFolder);
                    Response.Redirect(EditUrl("itemid", base.ItemId, base.CtrlTypeCode, param.ToArray()));
					break;
				case "catselect":
					UInfo.RtnUrl = Context.Request.RawUrl;
					UInfo.FromItemId = base.ItemId;
					UInfo.Save();
                    Response.Redirect(EditUrl("itemid", base.ItemId, "AdminCatXref", param.ToArray()));
					break;
				case "prdxrefselect":
					UInfo.RtnUrl = Context.Request.RawUrl;
					UInfo.FromItemId = base.ItemId;
					UInfo.Save();
                    Response.Redirect(EditUrl("itemid", base.ItemId, "AdminProductXref", param.ToArray()));
					break;
                case "prdimgselect":
                    UInfo.RtnUrl = Context.Request.RawUrl;
                    UInfo.FromItemId = base.ItemId;
                    UInfo.Save();
                    Response.Redirect(EditUrl("itemid", base.ItemId, "AdminProductImg", param.ToArray()));
                    break;
                case "prdimgadd":
                    UInfo.RtnUrl = Context.Request.RawUrl;
                    UInfo.FromItemId = base.ItemId;
                    UInfo.Save();
                    Response.Redirect(EditUrl("itemid", "-1", "AdminImages", param.ToArray()));
                    break;
                case "prdoptselect":
                    UInfo.RtnUrl = Context.Request.RawUrl;
                    UInfo.FromItemId = base.ItemId;
                    UInfo.Save();
                    Response.Redirect(EditUrl("itemid", base.ItemId, "AdminProductOpt", param.ToArray()));
                    break;
                case "prdoptadd":
                    UInfo.RtnUrl = Context.Request.RawUrl;
                    UInfo.FromItemId = base.ItemId;
                    UInfo.Save();
                    Response.Redirect(EditUrl("itemid", "-1", "AdminOptions", param.ToArray()));
                    break;
                case "exit":
					Response.Redirect(Globals.NavigateURL(PortalSettings.ActiveTab.TabID));
					break;
				case "saveexit":
					UpdateData();
					Response.Redirect(Globals.NavigateURL(PortalSettings.ActiveTab.TabID));
					break;
			}

		}


		protected virtual void EventCtrlEditItemCommand(object source, RepeaterCommandEventArgs e)
		{

		}

		protected void EventBeforeSelectLang()
		{
			UpdateData();
		}


		#region "presistance methods"

		protected virtual void EventAfterUpdateData(NBrightInfo objInfo)
		{

		}

		protected virtual void EventBeforeUpdateData()
		{

		}

		protected void UpdateData()
		{
			if (Page.IsValid)
			{
				EventBeforeUpdateData();

				//check upload folder exists
				Utils.CreateFolder(PortalSettings.HomeDirectoryMapPath + UploadFolder);

				#region "update lang field on repeater, if empty"

				var currentlang = GenXmlFunctions.GetHiddenField(rpLang, "lang");
				if (currentlang == "" | currentlang.ToLower() == "true") currentlang = Utils.GetCurrentCulture();
				GenXmlFunctions.SetHiddenField(rpLang, "lang", currentlang);

				#endregion

				// calc SEO name
				var seoname = GenXmlFunctions.GetField(rpData, "txtseoname");
				if (seoname == "") seoname = GenXmlFunctions.GetField(rpData, "txtname");
				var seonamelang = GenXmlFunctions.GetField(rpLang, "txtseoname");
				if (seonamelang == "") seonamelang = GenXmlFunctions.GetField(rpLang, "txtname");

				NBrightInfo objInfo = null;

				if (base.ItemId == "0" | base.ItemId == "")
				{
					//new record being create, so update the SEO name with the new id
					objInfo = (NBrightDNN.NBrightInfo)base.UpdateDetailNoValidate(rpData, EntityTypeCode, PortalSettings.HomeDirectoryMapPath + UploadFolder, base.ItemId, SelUserId, seoname);
					base.ItemId = objInfo.ItemID.ToString("");
					seoname = (seoname.Replace(" ", "-")).TrimEnd('-');
					seoname = Server.UrlEncode(Utils.StripAccents(seoname));
					objInfo = (NBrightDNN.NBrightInfo)base.UpdateDetailNoValidate(rpData, EntityTypeCode, PortalSettings.HomeDirectoryMapPath + UploadFolder, base.ItemId, SelUserId, seoname);
				}
				else
				{
					seoname = (seoname.Replace(" ", "-")).TrimEnd('-');
					seoname = Server.UrlEncode(Utils.StripAccents(seoname));
					// Does standard update of template form
					objInfo = (NBrightDNN.NBrightInfo)base.UpdateDetailNoValidate(rpData, EntityTypeCode, PortalSettings.HomeDirectoryMapPath + UploadFolder, base.ItemId, SelUserId, seoname);
				}

				#region "check if langauges enabled and update"

				if (rblSelectLang.Visible)
				{
					seonamelang = (objInfo.ItemID.ToString("") + "-" + seonamelang.Replace(" ", "-")).TrimEnd('-');
					seonamelang = Server.UrlEncode(Utils.StripAccents(seonamelang));

					var objInfoLang = base.UpdateDetailLangNoValidate(rpLang, base.EntityTypeCodeLang, PortalSettings.HomeDirectoryMapPath + UploadFolder, base.ItemIdLang, SelUserId, seonamelang);
					// copy to all empty langauges
					foreach (var l in NBrightDNN.DnnUtils.GetCultureCodeList())
					{
						var obj = GetData(l);
						if (obj.ItemID <= 0 & l != "")
						{
							base.CopyDataLang(base.EntityLangauge, l);
						}
					}
				}

				#endregion

				#region "Resync filesystem if required"

				if (FileHasBeenUploaded)
				{
					try
					{
						DotNetNuke.Services.FileSystem.FolderManager.Instance.Synchronize(PortalSettings.PortalId, UploadFolder + "/", true, true);
					}
					catch (Exception)
					{
						// stop error if sync fails...should be done by scheduler anyway.
					}
				}

				#endregion

				EventAfterUpdateData(objInfo);
			}
		}


		#endregion


		#region "Event Overrides"

		public override void EventBeforeBindListData(Repeater rpData, string webServiceUrl = "")
		{
			// Make sure that the NBrightStore uses -1 for all data search, data in the store is portal wide.
			UInfo.SearchModuleId = -1;
			UInfo.Save();

		}

		#endregion

	}

}
