﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Portals;
using NBrightCore.common;
using NBrightDNN;
using DotNetNuke.Modules.NBrightStore.Components;

namespace DotNetNuke.Modules.NBrightStore
{
    public static class CategoryUtils
    {

        #region "Category structure"

        public static List<NBrightInfo> GetTreeCatList(List<NBrightInfo> rtnList, List<NBrightInfo> catList, int level, int parentid, string prefix = "")
        {
            if (level > 20) // stop infinate loop
            {
                return rtnList;
            }
            if (parentid > 0) prefix += "..";
            foreach (NBrightDNN.NBrightInfo tInfo in catList)
            {
                var parenttestid = tInfo.ParentItemId;
                if (parenttestid < 0) parenttestid = 0;
                if (parentid == parenttestid)
                {
                    var nInfo = tInfo;
                    try
                    {
                        nInfo.SetXmlProperty("genxml/lang/genxml/hidden/breadcrumb", GetBreadCrumb(catList, nInfo.ItemID,7));
                        nInfo.SetXmlProperty("genxml/lang/genxml/textbox/txtname", prefix + "" + tInfo.GetXmlProperty("genxml/lang/genxml/textbox/txtname"));
                        rtnList.Add(nInfo);
                    }
                    catch (Exception)
                    {
                        nInfo.XMLData = "";
                        rtnList.Add(nInfo);
                    }
                    GetTreeCatList(rtnList, catList, level + 1, tInfo.ItemID, prefix);
                }
            }

            return rtnList;
        }

        public static List<NBrightInfo> GetCatList(String SqlFilter = "")
        {

            var objCtrl = new NBrightStoreController();
            var strFilter = SqlFilter;
            var strOrderBy = " order by [XMLData].value('(genxml/hidden/recordsortorder)[1]','nvarchar(10)'),[XMLData].value('(genxml/lang/genxml/textbox/txtname)[1]','nvarchar(50)') ";
            var catList = new List<NBrightDNN.NBrightInfo>();

            var l = objCtrl.GetListWithLang(PortalSettings.Current.PortalId, -1, "CATEGORY", strFilter, strOrderBy, 0, 0, 0, 0, "CATEGORYLANG", "");

            return l;

        }

        public static List<NBrightInfo> BuildCatList(String SqlFilter = "")
        {
            var l = GetCatList(SqlFilter);
            var rtnL = new List<NBrightInfo>();
            return GetTreeCatList(rtnL, l, 0, 0, "");
        }

        public static String GetParentCatIds(int CatItemid, String catIds = "")
        {
            var l = GetCatList();
            return GetParentCatIds(l, CatItemid, catIds);
        }

        public static String GetParentCatIds(List<NBrightInfo> catList, int Itemid, String catIds = "")
        {
            var lenum = from i in catList where i.ItemID == Itemid select i;
            var l = lenum.ToList();
            if (l.Any())
            {
                var strOut = l.First().ItemID + "*" + catIds;
                return GetParentCatIds(catList, l.First().ParentItemId, strOut.TrimEnd('*'));
            }
            return catIds;
        }


        public static String GetBreadCrumb(int CatItemid, int ShortLength, String BreadCrumb = "")
        {
            var l = GetCatList();
            return GetBreadCrumb(l, CatItemid, ShortLength, BreadCrumb);
        }

        public static String GetBreadCrumb(List<NBrightInfo> catList, int Itemid, int ShortLength, String BreadCrumb = "")
        {
            var lenum = from i in catList where i.ItemID == Itemid select i;
            var l = lenum.ToList();
            if (l.Any())
            {
                var crumbText = l.First().GetXmlProperty("genxml/lang/genxml/textbox/txtname");
                if (ShortLength > 0)
                {
                    if (crumbText.Length > (ShortLength + 1)) crumbText = crumbText.Substring(0, ShortLength) + ".";
                }

                var strOut = crumbText + ">" + BreadCrumb;
                return GetBreadCrumb(catList, l.First().ParentItemId, ShortLength, strOut.TrimEnd('>'));
            }
            return BreadCrumb;
        }

        public static String GetBreadCrumbWithLinks(int CatItemid, string defaultTabId, int ShortLength, String BreadCrumb = "")
        {
            var l = GetCatList();
            return GetBreadCrumbWithLinks(l, CatItemid, defaultTabId, ShortLength, BreadCrumb);
        }

        public static String GetBreadCrumbWithLinks(List<NBrightInfo> catList, int Itemid, string defaultTabId, int ShortLength, String BreadCrumb = "")
        {
            var lenum = from i in catList where i.ItemID == Itemid select i;
            var l = lenum.ToList();
            if (l.Any())
            {
                var crumbText = l.First().GetXmlProperty("genxml/lang/genxml/textbox/txtname");
                if (ShortLength > 0)
                {
                    if (crumbText.Length > (ShortLength + 1)) crumbText = crumbText.Substring(0, ShortLength) + ".";
                }
                
                var strOut = "<a href='" + GetCategoryUrl(l.First(),defaultTabId) + "'>" + crumbText + "</a>";
                if (BreadCrumb != "")
                {
                    strOut += ">";
                }
                strOut += BreadCrumb;
                return GetBreadCrumbWithLinks(catList, l.First().ParentItemId, defaultTabId, ShortLength, strOut);
            }
            return BreadCrumb;
        }


        public static int GetDefaultCatId(int EntryItemId)
        {
            var defId = 0;
            var objCtrl = new NBrightStoreController();
            var strFilter = " and parentitemid = " + EntryItemId.ToString("") + " ";
            var objInfo = objCtrl.GetInfo(EntryItemId);

            if (objInfo != null)
            {

                var l = objCtrl.GetListInfo(objInfo.PortalId, objInfo.ModuleId, "CATXREF", strFilter);
                foreach (var e in l)
                {
                    if (e.GetXmlProperty("genxml/hidden/defaultcat").ToLower() == "true")
                    {
                        defId = e.XrefItemId;
                        break;
                    }
                }

                if (defId == 0)
                {
                    if (l.Count > 0)
                    {
                        defId = l[0].XrefItemId;
                    }
                }
            }

            return defId;
        }

        public static string GetCategoryUrl(NBrightInfo categoryInfo, string DefaultTabId)
        {
            var objCtrl = new NBrightStoreController();
            var objLang = objCtrl.GetInfoLang(PortalSettings.Current.PortalId, categoryInfo.ModuleId, categoryInfo.ItemID.ToString(""), NBrightCore.common.Utils.GetCurrentCulture(), "CATEGORYLANG");

            var tabid = categoryInfo.GetXmlProperty("genxml/dropdownlist/cattabid");
            //get default tabid, if not set by category
            if (tabid == "")
            {
                tabid = DefaultTabId;
            }

            //set a default url
            var url = "?catid=" + categoryInfo.ItemID.ToString("");
            // get friendly url if possible
            if (objLang != null && NBrightCore.common.Utils.IsNumeric(tabid))
            {
				if (objLang.GetXmlProperty("genxml/textbox/txtname") != "" )
				{
					var newBaseName = objLang.GetXmlProperty("genxml/textbox/txtseoname");
					if (newBaseName == "") newBaseName = objLang.GetXmlProperty("genxml/textbox/txtname");
                    var tab = CBO.FillObject<DotNetNuke.Entities.Tabs.TabInfo>(DotNetNuke.Data.DataProvider.Instance().GetTab(Convert.ToInt32(tabid)));
                    if (tab != null)
                    {
						url = DotNetNuke.Services.Url.FriendlyUrl.FriendlyUrlProvider.Instance().FriendlyUrl(tab, "~/Default.aspx?TabId=" + tab.TabID.ToString("") + "&catid=" + categoryInfo.ItemID.ToString(""), newBaseName.Replace(" ","-") + ".aspx");
                    }
                }
            }
            return url;
        }

        public static NBrightInfo GetCurrentCategoryInfo(int portalId, int moduleId, System.Web.HttpRequest Request, int EntryId = 0)
        {
            NBrightInfo objCInfo = null;

            var defcatid = 0;
            var qrycatid = Utils.RequestQueryStringParam(Request, "catid");
            if (Utils.IsNumeric(EntryId))
            {
                if (EntryId > 0) defcatid = CategoryUtils.GetDefaultCatId(EntryId);
            }

            if (defcatid == 0)
            {
                if (Utils.IsNumeric(qrycatid))
                {
                    defcatid = Convert.ToInt32(qrycatid);
                }
            }

            if (defcatid == 0)
            {
                if (Utils.IsNumeric(portalId) | Utils.IsNumeric(moduleId))
                {
                    var nbSettings = NBrightStore.NBrightStoreUtils.GetSettings(Convert.ToInt32(portalId), Convert.ToInt32(moduleId));
                    var setcatid = nbSettings.GetXmlProperty("genxml/hidden/defaultcatid");
                    if (Utils.IsNumeric(setcatid))
                    {
                        defcatid = Convert.ToInt32(setcatid);
                    }
                }
            }
            var objCtrl = new NBrightStoreController();
            var objInfo = objCtrl.GetInfo(defcatid);
            if (objInfo != null)
            {
                objCInfo = objCtrl.GetDataWithLang(objInfo.PortalId, objInfo.ModuleId, objInfo.ItemID.ToString(""), Utils.GetCurrentCulture(), "CATEGORYLANG");
            }

            return objCInfo;
        }

        #endregion
    }
}