﻿using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Portals;
using NBrightCore.common;
using NBrightCore.providers;
using NBrightCore.render;
using DotNetNuke.Entities.Users;

namespace DotNetNuke.Modules.NBrightStore.Components
{
    public class GenXmlTemplateExt : GenXProvider
    {

        private string _rootname = "genxml";
        private string _databindColumn = "XMLData";

        #region "Override methods"

        // This section overrides the interface methods for the GenX provider.
        // It allows providers to create controls/Literals in the NBright template system.

        public override bool CreateGenControl(string ctrltype, Control container, XmlNode xmlNod, string rootname = "genxml", string databindColum = "XMLData")
        {
            _rootname = rootname;
            _databindColumn = databindColum;
            switch (ctrltype)
            {
                case "entrylink":
                    CreateEntryLink(container, xmlNod);
                    return true;
                case "returnlink":
                    CreateReturnLink(container, xmlNod);
                    return true;
                case "entryurl":
                    CreateEntryUrl(container, xmlNod);
                    return true;
                case "returnurl":
                    CreateReturnUrl(container, xmlNod);
                    return true;
                case "currenturl":
                    CreateCurrentUrl(container, xmlNod);
                    return true;
                case "catdropdown":
                    CreateDropDownList(container, xmlNod);
                    return true;
                case "catbreadcrumb":
                    CreateCatBreadCrumb(container, xmlNod);
                    return true;
                case "catshortcrumb":
                    CreateCatBreadCrumb(container, xmlNod);
                    return true;
                case "catdefaultname":
                    CreateCatDefaultName(container, xmlNod);
                    return true;
                case "catvalueof":
                    CreateCatValueOf(container, xmlNod);
                    return true;
                case "catbreakof":
                    CreateCatBreakOf(container, xmlNod);
                    return true;
                case "cathtmlof":
                    CreateCatHtmlOf(container, xmlNod);
                    return true;
                default:
                    return false;

            }

        }

        public override string GetField(Control ctrl)
        {
            return "";
        }

        public override void SetField(Control ctrl, string newValue)
        {
        }

        public override string GetGenXml(List<Control> genCtrls, XmlDataDocument xmlDoc, string originalXml, string folderMapPath, string xmlRootName = "genxml")
        {
            return "";
        }

        public override string GetGenXmlTextBox(List<Control> genCtrls, XmlDataDocument xmlDoc, string originalXml, string folderMapPath, string xmlRootName = "genxml")
        {
            return "";
        }

        public override object PopulateGenObject(List<Control> genCtrls, object obj)
        {
            return null;
        }

        #endregion


        #region "create EntryLink control"

        private void CreateEntryLink(Control container, XmlNode xmlNod)
        {
            var lk = new HyperLink();
            if (xmlNod.Attributes != null && (xmlNod.Attributes["text"] != null))
            {
                lk.Text = xmlNod.Attributes["text"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["cssclass"] != null))
            {
                lk.CssClass = xmlNod.Attributes["cssclass"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["class"] != null))
            {
                lk.CssClass = xmlNod.Attributes["class"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["target"] != null))
            {
                lk.Target = xmlNod.Attributes["target"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["imageurl"] != null))
            {
                lk.ImageUrl = xmlNod.Attributes["imageurl"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["tabid"] != null))
            {
                lk.Attributes.Add("tabid", xmlNod.Attributes["tabid"].InnerText);
            }

            lk.DataBinding += EntryLinkDataBinding;
            container.Controls.Add(lk);
        }

        private void EntryLinkDataBinding(object sender, EventArgs e)
        {
            var lk = (HyperLink)sender;
            var container = (IDataItemContainer)lk.NamingContainer;
			try
			{
				//set a default url


				var entryid = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemID"));
				var moduleid = Convert.ToString(DataBinder.Eval(container.DataItem, "ModuleId"));
				var guidkey = Convert.ToString(DataBinder.Eval(container.DataItem, "GUIDKey"));

				var nod = GenXmlFunctions.GetGenXmLnode(DataBinder.Eval(container.DataItem, _databindColumn).ToString(), "genxml/lang/genxml/textbox/txttitle");
				if ((nod != null))
				{
					guidkey = nod.InnerText;
				}
				if (guidkey == "")
				{
					nod = GenXmlFunctions.GetGenXmLnode(DataBinder.Eval(container.DataItem, _databindColumn).ToString(), "genxml/lang/genxml/textbox/txtname");
					if ((nod != null))
					{
						guidkey = nod.InnerText;
					}
				}
				if (guidkey == "")
				{
					nod = GenXmlFunctions.GetGenXmLnode(DataBinder.Eval(container.DataItem, _databindColumn).ToString(), "genxml/textbox/txttitle");
					if ((nod != null))
					{
						guidkey = nod.InnerText;
					}
				}

				if (guidkey == "")
				{
					nod = GenXmlFunctions.GetGenXmLnode(DataBinder.Eval(container.DataItem, _databindColumn).ToString(), "genxml/textbox/txtname");
					if ((nod != null))
					{
						guidkey = nod.InnerText;
					}
				}

				var t = "";
				if (lk.Attributes["tabid"] != null && Utils.IsNumeric(lk.Attributes["tabid"]))
				{
					t = lk.Attributes["tabid"];
				}

				var url = GetEntryUrl(entryid, moduleid, guidkey, t);
				lk.NavigateUrl = url;

			}
			catch (Exception ex)
			{
				lk.Text = ex.ToString();
			}
        }

        private void CreateEntryUrl(Control container, XmlNode xmlNod)
        {
            var l = new Literal();

            l.DataBinding += EntryUrlDataBinding;
            container.Controls.Add(l);
        }

        private void EntryUrlDataBinding(object sender, EventArgs e)
        {
            var l = (Literal)sender;
            var container = (IDataItemContainer)l.NamingContainer;
            try
            {
                //set a default url
                var entryid = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemID"));
                var moduleid = Convert.ToString(DataBinder.Eval(container.DataItem, "ModuleId"));
                var guidkey = Convert.ToString(DataBinder.Eval(container.DataItem, "GUIDKey"));

                var nod = GenXmlFunctions.GetGenXmLnode(DataBinder.Eval(container.DataItem, _databindColumn).ToString(), "genxml/lang/guidkey");
                if ((nod != null))
                {
                    guidkey = nod.InnerText;
                }

                var url = GetEntryUrl(entryid, moduleid, guidkey, "");
                l.Text = url;

            }
            catch (Exception ex)
            {
                l.Text = ex.ToString();
            }
        }


        #endregion

        #region "create ReturnLink control"

        private void CreateReturnLink(Control container, XmlNode xmlNod)
        {
            var lk = new HyperLink();
            if (xmlNod.Attributes != null && (xmlNod.Attributes["text"] != null))
            {
                lk.Text = xmlNod.Attributes["text"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["cssclass"] != null))
            {
                lk.CssClass = xmlNod.Attributes["cssclass"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["class"] != null))
            {
                lk.CssClass = xmlNod.Attributes["class"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["target"] != null))
            {
                lk.Target = xmlNod.Attributes["target"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["imageurl"] != null))
            {
                lk.ImageUrl = xmlNod.Attributes["imageurl"].InnerXml;
            }
            if (xmlNod.Attributes != null && (xmlNod.Attributes["tabid"] != null))
            {
                lk.Attributes.Add("tabid", xmlNod.Attributes["tabid"].InnerText);
            }

            lk.DataBinding += ReturnLinkDataBinding;
            container.Controls.Add(lk);
        }

        private void ReturnLinkDataBinding(object sender, EventArgs e)
        {
            var lk = (HyperLink)sender;
            var container = (IDataItemContainer)lk.NamingContainer;
            try
            {
                //set a default url
                var entryid = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemID"));
                var moduleid = Convert.ToString(DataBinder.Eval(container.DataItem, "ModuleId"));
                var guidkey = Convert.ToString(DataBinder.Eval(container.DataItem, "GUIDKey"));

                var t = "";
                if (lk.Attributes["tabid"] != null && Utils.IsNumeric(lk.Attributes["tabid"]))
                {
                    t = lk.Attributes["tabid"];
                }

                var url = GetReturnUrl(moduleid, t);
                lk.NavigateUrl = url;

            }
            catch (Exception ex)
            {
                lk.Text = ex.ToString();
            }
        }

        private void CreateReturnUrl(Control container, XmlNode xmlNod)
        {
            var l = new Literal();

            l.DataBinding += ReturnUrlDataBinding;
            container.Controls.Add(l);
        }

        private void ReturnUrlDataBinding(object sender, EventArgs e)
        {
            var l = (Literal)sender;
            var container = (IDataItemContainer)l.NamingContainer;
            try
            {
                //set a default url
                var entryid = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemID"));
                var moduleid = Convert.ToString(DataBinder.Eval(container.DataItem, "ModuleId"));
                var guidkey = Convert.ToString(DataBinder.Eval(container.DataItem, "GUIDKey"));

                var url = GetEntryUrl(entryid, moduleid, guidkey, "");
                l.Text = url;

            }
            catch (Exception ex)
            {
                l.Text = ex.ToString();
            }
        }

        #endregion

        #region "create CurrentUrl control"
        private void CreateCurrentUrl(Control container, XmlNode xmlNod)
        {
            var l = new Literal();

            l.DataBinding += CurrentUrlDataBinding;
            container.Controls.Add(l);
        }

        private void CurrentUrlDataBinding(object sender, EventArgs e)
        {
            var l = (Literal)sender;
            var container = (IDataItemContainer)l.NamingContainer;
            try
            {
                //set a default url
                var url = DotNetNuke.Entities.Portals.PortalSettings.Current.ActiveTab.FullUrl;
                l.Text = url;

            }
            catch (Exception ex)
            {
                l.Text = ex.ToString();
            }
        }

        #endregion

        #region "Build Urls"

        private string GetEntryUrl(string entryid,string moduleid,string guidkey,string tabid)
        {
            var redirectTab = DotNetNuke.Entities.Portals.PortalSettings.Current.ActiveTab;
            var rdTabid = redirectTab.TabID;

            NBrightDNN.NBrightInfo NbSettings = null;
            if (Utils.IsNumeric(moduleid))
            {
                NbSettings = NBrightStoreUtils.GetSettings(PortalSettings.Current.PortalId, Convert.ToInt32(moduleid));
            }

            if (NbSettings != null)
            {
                var temp = NbSettings.GetXmlProperty("genxml/dropdownlist/ddldetailtabid");
                if (Utils.IsNumeric(temp))
                {
                    if (Convert.ToInt32(temp) > 0)
                    {
                        rdTabid = Convert.ToInt32(temp);
                    }
                }
            }

            if (Utils.IsNumeric(tabid))
            {
                if (Convert.ToInt32(tabid) > 0)
                {
                    rdTabid = Convert.ToInt32(tabid);
                }
            }

            if (Utils.IsNumeric(rdTabid))
            {
                if (Convert.ToInt32(rdTabid) != redirectTab.TabID)
                {
                    var portalTabs = NBrightDNN.DnnUtils.GetPortalTabs(PortalSettings.Current.PortalId);
                    if (portalTabs != null)
                    {
                        var rTab = portalTabs[Convert.ToInt32(rdTabid)];
                        if (rTab != null)
                        {
                            redirectTab = rTab;
                        }
                    }
                }
            }

            return DotNetNuke.Services.Url.FriendlyUrl.FriendlyUrlProvider.Instance().FriendlyUrl(redirectTab, "~/Default.aspx?TabId=" + rdTabid.ToString("") + "&eid=" + entryid + "&modid=" + moduleid, guidkey.Replace(entryid + "-", "") + ".aspx");

        }

        private string GetReturnUrl(string moduleid, string tabid)
        {

            var redirectTab = DotNetNuke.Entities.Portals.PortalSettings.Current.ActiveTab;
            var rdTabid = redirectTab.TabID;

            NBrightDNN.NBrightInfo NbSettings = null;
            if (Utils.IsNumeric(moduleid))
            {
                NbSettings = NBrightStoreUtils.GetSettings(PortalSettings.Current.PortalId, Convert.ToInt32(moduleid));
            }

            if (NbSettings != null)
            {
                var temp = NbSettings.GetXmlProperty("genxml/dropdownlist/ddllisttabid");
                if (Utils.IsNumeric(temp))
                {
                    if (Convert.ToInt32(temp) > 0)
                    {
                        rdTabid = Convert.ToInt32(temp);
                    }
                }
            }

            if (Utils.IsNumeric(tabid))
            {
                if (Convert.ToInt32(tabid) > 0)
                {
                    rdTabid = Convert.ToInt32(tabid);
                }
            }

            if (Utils.IsNumeric(rdTabid))
            {
                if (Convert.ToInt32(rdTabid) != redirectTab.TabID)
                {
                    var portalTabs = NBrightDNN.DnnUtils.GetPortalTabs(PortalSettings.Current.PortalId);
                    if (portalTabs != null)
                    {
                        var rTab = portalTabs[Convert.ToInt32(rdTabid)];
                        if (rTab != null)
                        {
                            redirectTab = rTab;
                        }
                    }
                }
            }

            return DotNetNuke.Services.Url.FriendlyUrl.FriendlyUrlProvider.Instance().FriendlyUrl(redirectTab, "~/Default.aspx?TabId=" + redirectTab.TabID.ToString(""));

        }

        #endregion

        #region  "catdropdown"

        private void CreateDropDownList(Control container, XmlNode xmlNod)
        {
            var ddl = new DropDownList();
            ddl = (DropDownList)GenXmlFunctions.AssignByReflection(ddl, xmlNod);

            if (xmlNod.Attributes != null && (xmlNod.Attributes["datacats"] != null))
            {
                ddl.Attributes.Add("datacats", xmlNod.Attributes["datacats"].InnerXml);
                var tList = BuildCatList();

                if (xmlNod.Attributes["datacats"].InnerXml.ToLower() == "blank")
                {
                    var li = new ListItem();
                    li.Text = "";
                    li.Value = "";
                    ddl.Items.Add(li);
                }

                foreach (var tItem in tList)
                {
                    var li = new ListItem();
                    li.Text = tItem.Value;
                    li.Value = tItem.Key.ToString("");
                    ddl.Items.Add(li);
                }
            }

            ddl.DataBinding += DdListDataBinding;
            container.Controls.Add(ddl);
        }

        private void DdListDataBinding(object sender, EventArgs e)
        {
            var ddl = (DropDownList)sender;
            var container = (IDataItemContainer)ddl.NamingContainer;

            try
            {
                var strValue = GenXmlFunctions.GetGenXmlValue(ddl.ID, "dropdownlist", Convert.ToString(DataBinder.Eval(container.DataItem, _databindColumn)));

                if ((ddl.Items.FindByValue(strValue) != null))
                {
                    ddl.SelectedValue = strValue;
                }
                else
                {
                    var nod = GenXmlFunctions.GetGenXmLnode(ddl.ID, "dropdownlist", Convert.ToString(DataBinder.Eval(container.DataItem, _databindColumn)));
                    if ((nod.Attributes != null) && (nod.Attributes["selectedtext"] != null))
                    {
                        strValue = XmlConvert.DecodeName(nod.Attributes["selectedtext"].Value);
                        if ((ddl.Items.FindByValue(strValue) != null))
                        {
                            ddl.SelectedValue = strValue;
                        }
                    }
                }
            }
            catch (Exception)
            {
                //do nothing
            }
        }

        private Dictionary<int, string> GetTreeCatList(Dictionary<int, string> rtnList, List<NBrightDNN.NBrightInfo> catList, int level, int parentid, string prefix = "")
        {
            if (level > 20) // stop infinate loop
            {
                return rtnList;
            }
            if (parentid > 0) prefix += "..";
            foreach (NBrightDNN.NBrightInfo tInfo in catList)
            {
                var parenttestid = tInfo.ParentItemId;
                if (parenttestid < 0) parenttestid = 0;
                if (parentid == parenttestid)
                {
                    rtnList.Add(tInfo.ItemID, prefix + "" + tInfo.GetXmlProperty("genxml/lang/genxml/textbox/txtname"));
                    GetTreeCatList(rtnList, catList, level + 1, tInfo.ItemID, prefix);
                }
            }

            return rtnList;
        }

        private Dictionary<int, string> BuildCatList()
        {

            var objCtrl = new NBrightStoreController();
            var strFilter = "";
            var strOrderBy = " order by [XMLData].value('(genxml/hidden/sortorder)[1]','nvarchar(10)'),[XMLData].value('(genxml/lang/genxml/textbox/txtname)[1]','nvarchar(50)') ";
            var catList = new List<NBrightDNN.NBrightInfo>();

            var l = objCtrl.GetListWithLang(PortalSettings.Current.PortalId, -1, "CATEGORY", strFilter, strOrderBy, 0, 0, 0, 0, "CATEGORYLANG", "");

            var d = new Dictionary<int, string>();
            return GetTreeCatList(d, l, 0, 0, "");

        }

        #endregion

        #region "catbreadcrumb"

        private void CreateCatBreadCrumb(Control container, XmlNode xmlNod)
        {
            var lc = new Literal();
            lc.Text = xmlNod.OuterXml;
            lc.DataBinding += CatBreadCrumbDataBind;
            container.Controls.Add(lc);
        }

        private void CatBreadCrumbDataBind(object sender, EventArgs e)
        {
            var lc = (Literal)sender;
            var container = (IDataItemContainer)lc.NamingContainer;
            try
            {
                var moduleId = DataBinder.Eval(container.DataItem, "ModuleId");
                var id = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemId"));
                if (Utils.IsNumeric(id))
                {
                    var objCInfo = (NBrightDNN.NBrightInfo)container.DataItem;
                    if (objCInfo.TypeCode == "DATA")
                    {
                        // Is EntryId, so Get default category for entry.
                        objCInfo = CategoryUtils.GetCurrentCategoryInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), lc.Page.Request, Convert.ToInt32(id));
                    }
                    if (objCInfo != null)
                    {
                        var xmlDoc = new XmlDataDocument();
                        xmlDoc.LoadXml("<root>" + lc.Text + "</root>");
                        var xmlNod = xmlDoc.SelectSingleNode("root/tag");
                        var intLength = 400;
                        var intShortLength = -1;
                        var IsLink = false;

                        if (xmlNod.Attributes != null)
                        {
                            if (xmlNod.Attributes["length"] != null)
                            {
                                if (Utils.IsNumeric(xmlNod.Attributes["length"].InnerText))
                                {
                                    intLength = Convert.ToInt32(xmlNod.Attributes["length"].InnerText);
                                }
                            }
                            if (xmlNod.Attributes["links"] != null) IsLink = true;
                            if (xmlNod.Attributes["short"] != null)
                            {
                                if (Utils.IsNumeric(xmlNod.Attributes["short"].InnerText))
                                {
                                    intShortLength = Convert.ToInt32(xmlNod.Attributes["short"].InnerText);
                                }
                            }
                        }

                        var defcatid = objCInfo.ItemID;
                        if (IsLink)
                        {
                            if (Utils.IsNumeric(moduleId))
                            {
                                var nbSettings = NBrightStore.NBrightStoreUtils.GetSettings(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId));
                                var defTabId = nbSettings.GetXmlProperty("genxml/dropdownlist/cattabid");
                                lc.Text = CategoryUtils.GetBreadCrumbWithLinks(defcatid, defTabId, intShortLength);
                            }
                        }
                        else
                        {
                            lc.Text = CategoryUtils.GetBreadCrumb(defcatid, intShortLength);
                        }

                        if (lc.Text.Length > intLength)
                        {
                            lc.Text = lc.Text.Substring(0, (intLength - 3)) + "...";
                        }
                    }
                }
            }
            catch (Exception)
            {
                lc.Text = "";
            }
        }

        #endregion

        #region "CreateCatDefaultName"

        private void CreateCatDefaultName(Control container, XmlNode xmlNod)
        {
            var lc = new Literal();
            lc.DataBinding += CatDefaultNameDataBind;
            container.Controls.Add(lc);
        }

        private void CatDefaultNameDataBind(object sender, EventArgs e)
        {
            var lc = (Literal)sender;
            lc.Text = "";
            var container = (IDataItemContainer)lc.NamingContainer;
            try
            {
                var moduleId = DataBinder.Eval(container.DataItem, "ModuleId");
                var id = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemId"));
                if (Utils.IsNumeric(id) && Utils.IsNumeric(moduleId))
                {
                    var objCInfo = CategoryUtils.GetCurrentCategoryInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), lc.Page.Request, Convert.ToInt32(id));
                    if (objCInfo != null)
                    {
                        lc.Text = objCInfo.GetXmlProperty("genxml/lang/genxml/textbox/txtname");
                    }
                }

            }
            catch (Exception ex)
            {
                lc.Text = "";
            }
        }

        #endregion

        #region "CreateCatValueOf"

        private void CreateCatValueOf(Control container, XmlNode xmlNod)
        {
            var lc = new Literal();
            if (xmlNod.Attributes != null && (xmlNod.Attributes["xpath"] != null))
            {
                lc.Text = xmlNod.Attributes["xpath"].Value;
            }
            lc.DataBinding += CatValueOfDataBind;
            container.Controls.Add(lc);
        }

        private void CatValueOfDataBind(object sender, EventArgs e)
        {
            var lc = (Literal)sender;
            var container = (IDataItemContainer)lc.NamingContainer;
            try
            {
                var moduleId = DataBinder.Eval(container.DataItem, "ModuleId");
                var id = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemId"));
                if (Utils.IsNumeric(id) && Utils.IsNumeric(moduleId))
                {
                    var objCInfo = CategoryUtils.GetCurrentCategoryInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), lc.Page.Request, Convert.ToInt32(id));
                    if (objCInfo != null)
                    {
                        lc.Text = objCInfo.GetXmlProperty(lc.Text);
                    }
                }

            }
            catch (Exception ex)
            {
                lc.Text = "";
            }
        }

        #endregion

        #region "CreateCatBreakOf"

        private void CreateCatBreakOf(Control container, XmlNode xmlNod)
        {
            var lc = new Literal();
            if (xmlNod.Attributes != null && (xmlNod.Attributes["xpath"] != null))
            {
                lc.Text = xmlNod.Attributes["xpath"].Value;
            }
            lc.DataBinding += CatBreakOfDataBind;
            container.Controls.Add(lc);
        }

        private void CatBreakOfDataBind(object sender, EventArgs e)
        {
            var lc = (Literal)sender;
            var container = (IDataItemContainer)lc.NamingContainer;
            try
            {
                var moduleId = DataBinder.Eval(container.DataItem, "ModuleId");
                var id = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemId"));
                if (Utils.IsNumeric(id) && Utils.IsNumeric(moduleId))
                {
                    var objCInfo = CategoryUtils.GetCurrentCategoryInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), lc.Page.Request, Convert.ToInt32(id));
                    if (objCInfo != null)
                    {
                        lc.Text = objCInfo.GetXmlProperty(lc.Text);
                    }
                    lc.Text = System.Web.HttpUtility.HtmlEncode(lc.Text);
                    lc.Text = lc.Text.Replace(Environment.NewLine, "<br/>");
                }

            }
            catch (Exception ex)
            {
                lc.Text = "";
            }
        }

        #endregion

        #region "CreateCatHtmlOf"

        private void CreateCatHtmlOf(Control container, XmlNode xmlNod)
        {
            var lc = new Literal();
            if (xmlNod.Attributes != null && (xmlNod.Attributes["xpath"] != null))
            {
                lc.Text = xmlNod.Attributes["xpath"].Value;
            }
            lc.DataBinding += CatHtmlOfDataBind;
            container.Controls.Add(lc);
        }

        private void CatHtmlOfDataBind(object sender, EventArgs e)
        {
            var lc = (Literal)sender;
            var container = (IDataItemContainer)lc.NamingContainer;
            try
            {
                var moduleId = DataBinder.Eval(container.DataItem, "ModuleId");
                var id = Convert.ToString(DataBinder.Eval(container.DataItem, "ItemId"));
                if (Utils.IsNumeric(id) && Utils.IsNumeric(moduleId))
                {
                    var objCInfo = CategoryUtils.GetCurrentCategoryInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), lc.Page.Request, Convert.ToInt32(id));
                    if (objCInfo != null)
                    {
                        lc.Text = objCInfo.GetXmlProperty(lc.Text);
                    }
                    lc.Text = System.Web.HttpUtility.HtmlDecode(lc.Text);
                }

            }
            catch (Exception ex)
            {
                lc.Text = "";
            }
        }

        #endregion



    }
}
