﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Web;
using System.Web.UI.WebControls;
using System.Xml;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Entities.Portals;
using DotNetNuke.Services.Search;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;

namespace DotNetNuke.Modules.NBrightStore.Components
{

	public class NBrightStoreController : DataCtrlInterface, IPortable, ISearchable
	{

		#region "NBrightGen override DB Public Methods"

		public override void DeleteInfo(int ItemID)
		{
			DataProvider.Instance().Delete(ItemID);
		}

		public override void CleanData()
		{
			DataProvider.Instance().CleanData();
		}

		public override NBrightInfo GetInfo(int ItemID)
		{
			return CBO.FillObject<NBrightInfo>(DataProvider.Instance().Get(ItemID));
		}

		public override List<NBrightInfo> GetListInfo(int PortalId, int ModuleId, string TypeCode, string SQLSearchFilter = "", string SQLOrderBy = "", int ReturnLimit = 0, int PageNumber = 0, int PageSize = 0, int RecordCount = 0, string TypeCodeLang = "")
		{
			return CBO.FillCollection<NBrightInfo>(DataProvider.Instance().GetList(PortalId, ModuleId, TypeCode, SQLSearchFilter, SQLOrderBy, ReturnLimit, PageNumber, PageSize, RecordCount, TypeCodeLang));
		}

		public override int GetListInfoCount(int PortalId, int ModuleId, string TypeCode, string SQLSearchFilter = "", string TypeCodeLang = "")
		{
			return DataProvider.Instance().GetListCount(PortalId, ModuleId, TypeCode, SQLSearchFilter, TypeCodeLang);
		}

		public override int UpdateInfo(NBrightInfo objInfo)
		{
			objInfo.ModifiedDate = DateTime.Now;
			return DataProvider.Instance().Update(objInfo.ItemID, objInfo.PortalId, objInfo.ModuleId, objInfo.TypeCode, objInfo.XMLData, objInfo.GUIDKey, objInfo.ModifiedDate, objInfo.TextData, objInfo.XrefItemId, objInfo.ParentItemId, objInfo.UserId, objInfo.Lang);
		}

		public override NBrightInfo GetInfoByType(int PortalId, int ModuleId, string EntityTypeCode, string selUserId = "", string lang = "")
		{
			var strFilter = "";
			if (selUserId != "")
			{
				strFilter += " and UserId = " + selUserId + " ";
			}
			if (lang != "")
			{
				strFilter += " and lang = '" + lang + "' ";
			}

			var l = CBO.FillCollection<NBrightInfo>(DataProvider.Instance().GetList(PortalId, ModuleId, EntityTypeCode, strFilter, "", 1, 1, 1, 1, ""));
			if (l.Count >= 1)
			{
				return l[0];
			}
			return null;
		}

		public override NBrightInfo GetInfoByGuidKey(int PortalId, int ModuleId, string EntityTypeCode, string GuidKey, string selUserId = "")
		{
			var strFilter = " and GUIDKey = '" + GuidKey + "' ";
			if (selUserId != "")
			{
				strFilter += " and UserId = " + selUserId + " ";
			}

			var l = CBO.FillCollection<NBrightInfo>(DataProvider.Instance().GetList(PortalId, ModuleId, EntityTypeCode, strFilter, "", 1, 1, 1, 1, ""));
			if (l.Count == 0)
			{
				return null;
			}
			if (l.Count > 1)
			{
				for (int i = 1; i < l.Count; i++)
				{
					// remove invalid DB entries
					DeleteInfo(l[i].ItemID);
				}
			}
			return l[0];
		}

		/* *********************  Data Gets ********************** */

		public NBrightInfo GetInfoByTypeUser(int PortalId, int ModuleId, string EntityTypeCode, string selUserId)
		{
			var strFilter = "";
			if (selUserId != "")
			{
				strFilter = " and UserId = " + selUserId + " ";
			}
			var l = CBO.FillCollection<NBrightInfo>(DataProvider.Instance().GetList(PortalId, ModuleId, EntityTypeCode, strFilter, "", 1, 1, 1, 1, ""));
			if (l.Count >= 1)
			{
				return l[0];
			}
			return null;
		}

		public NBrightInfo GetInfoLang(int PortalId, int ModuleId, string parentItemId, string cultureCode, string entityTypeCodeLang, string selUserId = "")
		{
			NBrightInfo rtnObj = null;
			if (Utils.IsNumeric(parentItemId))
			{
				var strFilter = " and (Lang = '" + cultureCode + "' or ISNULL(Lang,'') = '' or [xmlData].value('(genxml/hidden/lang)[1]', 'nvarchar(10)') = '" + cultureCode + "' or [xmlData].value('(genxml/hidden/lang)[1]', 'nvarchar(10)') = '' ) ";
				if (Utils.IsNumeric(parentItemId))
				{
					strFilter = " and parentitemid = " + parentItemId + strFilter;
				}
				if (selUserId != "")
				{
					strFilter += " and UserId = " + selUserId + " ";
				}
				var l = GetListInfo(PortalId, ModuleId, entityTypeCodeLang, strFilter, "", 1);
				if (l != null)
				{
					if (l.Count > 0)
					{
						rtnObj = l[0];
					}
				}
			}
			return rtnObj;
		}

		public override NBrightInfo GetDataWithLang(int PortalId, int ModuleId, string parentItemId, string cultureCode, string entityTypeCodeLang, string selUserId = "")
		{
			NBrightInfo rtnObj = null;
			if (Utils.IsNumeric(parentItemId))
			{
				rtnObj = GetInfo(Convert.ToInt32(parentItemId));
				var oLang = GetInfoLang(PortalId, ModuleId, parentItemId, cultureCode, entityTypeCodeLang, selUserId);
				if (oLang != null && oLang.XMLData != null)
				{
					var xmlDoc = new XmlDataDocument();
					xmlDoc.LoadXml(rtnObj.XMLData);

					// create xml to be inserted
					var strLangXML = "<lang><guidkey>" + oLang.GUIDKey + "</guidkey>" + oLang.XMLData + "</lang>"; // Add Langauge xml

					// Create a document fragment to contain the XML to be inserted. 
					XmlDocumentFragment docFrag = xmlDoc.CreateDocumentFragment();
					// Set the contents of the document fragment. 
					docFrag.InnerXml = strLangXML;
					//Add new model data
					if (xmlDoc.DocumentElement != null) xmlDoc.DocumentElement.AppendChild(docFrag);

					rtnObj.XMLData = xmlDoc.OuterXml;
				}
			}
			return rtnObj;
		}

		/* *********************  list Data Gets ********************** */

		public override List<NBrightInfo> GetListWithLang(Repeater rp1, int portalId, int moduleId, string typeCode, int returnLimit = 0, int pageNumber = 0, int pageSize = 0, string entityTypeCodeLang = "", string selUserId = "", bool debugMode = false)
		{
			var strFilters = GenXmlFunctions.GetSqlSearchFilters(rp1);
			var strOrderBy = GenXmlFunctions.GetSqlOrderBy(rp1);
			//Default orderby if not set
			if (String.IsNullOrEmpty(strOrderBy)) strOrderBy = " Order by ModifiedDate DESC ";
			return GetListWithLang(portalId, moduleId, typeCode, strFilters, strOrderBy, returnLimit, pageNumber, pageSize, 0, entityTypeCodeLang, selUserId, debugMode);
		}


		public override List<NBrightInfo> GetListWithLang(int portalId, int moduleId, string typeCode, string strFilters, string strOrderBy, int returnLimit = 0, int pageNumber = 0, int pageSize = 0, int recordCount = 0, string entityTypeCodeLang = "", string selUserId = "", bool debugMode = false)
		{
			if (selUserId != "")
			{
				strFilters += " and UserId = " + selUserId + " ";
			}

			List<NBrightInfo> l = null;

			// get cache template
			var strCacheKey = portalId.ToString() + "*" + moduleId.ToString() + "*" + typeCode + "*" + "*filter:" + strFilters.Replace(" ", "") + "*orderby:" + strFilters.Replace(" ", "") + "*" + returnLimit.ToString() + "*" + pageNumber.ToString() + "*" + pageSize.ToString() + "*" + recordCount.ToString() + "*" + entityTypeCodeLang.ToString();
			if (debugMode == false)
			{
				l = (List<NBrightInfo>)NBrightCore.common.Utils.GetCache(strCacheKey);
			}

			if (l == null)
			{
				l = GetListInfo(portalId, moduleId, typeCode, strFilters, strOrderBy, returnLimit, pageNumber, pageSize, recordCount, entityTypeCodeLang);
				if (!String.IsNullOrEmpty(entityTypeCodeLang))
				{
					int lp = 0;
					foreach (var o in l)
					{
						lp = lp + 1;
						o.RowCount = lp;
						var oLang = GetDataWithLang(portalId, moduleId, o.ItemID.ToString(""), Utils.GetCurrentCulture(), entityTypeCodeLang, selUserId);
						if (oLang != null && oLang.XMLData != null)
						{
							// replace the non-lang xmldata with the xmldata with the added langauge
							o.XMLData = oLang.XMLData;
						}
					}
				}
				//NBrightStoreUtils.SetModCache(moduleId, strCacheKey, l);
			}
			return l;
		}


        public string ReplaceUrlTokens(string strTemplate)
        {
            const string tokenTag = "Url:";
            if (strTemplate.Contains(tokenTag))
            {
                var aryTempl = Utils.ParseTemplateText(strTemplate);
                foreach (var s in aryTempl)
                {
                    if (s.StartsWith(tokenTag))
                    {
                        var urlparam = s.Replace(tokenTag, "").Replace("]", "");
                        strTemplate = strTemplate.Replace(s, Utils.RequestParam(HttpContext.Current, urlparam));
                    }
                }
            }
            return strTemplate;
        }



		#endregion


		#region Optional Interfaces

		#region IPortable Members

		/// -----------------------------------------------------------------------------
		/// <summary>
		///   ExportModule implements the IPortable ExportModule Interface
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <param name = "moduleId">The Id of the module to be exported</param>
		/// <history>
		/// </history>
		/// -----------------------------------------------------------------------------
		public string ExportModule(int ModuleId)
		{
			var objModCtrl = new ModuleController();
			var xmlOut = "";

			var objModInfo = objModCtrl.GetModule(ModuleId);

			if (objModInfo != null)
			{
				var portalId = objModInfo.PortalID;

				xmlOut += "<root>";

				xmlOut += "</root>";
			}

			return xmlOut;
		}

		/// -----------------------------------------------------------------------------
		/// <summary>
		///   ImportModule implements the IPortable ImportModule Interface
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <param name = "ModuleID">The ID of the Module being imported</param>
		/// <param name = "Content">The Content being imported</param>
		/// <param name = "Version">The Version of the Module Content being imported</param>
		/// <param name = "UserId">The UserID of the User importing the Content</param>
		/// <history>
		/// </history>
		/// -----------------------------------------------------------------------------

		public void ImportModule(int ModuleID, string Content, string Version, int UserId)
		{
			var xmlDoc = new XmlDataDocument();
			var objModCtrl = new ModuleController();
			var objModInfo = objModCtrl.GetModule(ModuleID);
			if (objModInfo != null)
			{

				xmlDoc.LoadXml(Content);

			}

		}

		#endregion


		#region ISearchable Members

		/// -----------------------------------------------------------------------------
		/// <summary>
		///   GetSearchItems implements the ISearchable Interface
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <param name = "ModInfo">The ModuleInfo for the module to be Indexed</param>
		/// <history>
		/// </history>
		/// -----------------------------------------------------------------------------
		public SearchItemInfoCollection GetSearchItems(ModuleInfo ModInfo)
		{
			var searchItemCollection = new SearchItemInfoCollection();
			return searchItemCollection;
		}

		#endregion


		#endregion

	}

}
