﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Runtime.Remoting.Contexts;
using System.Security.Cryptography;
using System.Web;
using System.Xml;
using DotNetNuke.Common;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Modules.NBrightStore.Components;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;

namespace DotNetNuke.Modules.NBrightStore
{
    public static class NBrightStoreUtils
    {
        public enum ModuleEventCodes { none, displaycategoryheader, displaycategorybody, displaycategoryfooter, displayentryheader, displayentrybody, displayentryfooter, displayheader, displaybody, displayfooter, selectsearch, selectheader, selectbody, selectfooter, selectedheader, selectedbody, selectedfooter, editheader, editbody, editlang, editfooter, editlistsearch, editlistheader, editlistbody, editlistfooter, email, emailsubject, emailclient, emailreturnmsg, jsinsert, exportxsl };

        public static NBrightInfo GetSettings(int PortalId, int ModuleId)
        {
            var obj = (NBrightInfo)NBrightCore.common.Utils.GetCache("NBright_NBsettings" + PortalId.ToString("") + "_" + ModuleId.ToString(""));
            if (obj == null)
            {
                // single record for EntityTypeCode settings, so get record directly.
                var objCtrl = new NBrightStoreController();
                obj = objCtrl.GetInfoByType(PortalId, ModuleId, "SETTINGS");
                if (obj == null)
                {
                    obj = new NBrightInfo { ItemID = -1 };
                    NBrightCore.common.Utils.SetCache("NBright_NBsettings" + PortalId.ToString("") + "_" + ModuleId.ToString(""), obj);
                }
            }
            return obj;
        }

        public static string FormatListtoXml(IEnumerable<NBrightInfo> objList)
        {
            var xmlOut = "";
            foreach (var obj in objList)
            {
                if (obj.XMLData.StartsWith("<root>"))
                {
                    // Data is Xref, so read actual data required.
                    var xrefId = GenXmlFunctions.GetGenXmlValue(obj.XMLData, "root/xref");
                    if (Utils.IsNumeric(xrefId))
                    {
                        var objCtrl = new Components.NBrightStoreController();
                        var obj2 = objCtrl.GetInfo(Convert.ToInt32(xrefId));
                        if (obj2 != null)
                        {
                            var linedxrefId = GenXmlFunctions.GetGenXmlValue(obj.XMLData, "root/linkedxref");
                            xmlOut += "<item id=\"" + obj.ItemID.ToString("") + "\" idxref=\"" + linedxrefId + "\" modifieddate=\"" + obj.ModifiedDate.ToString("s") + "\">" + obj2.XMLData + "</item>";
                        }
                    }
                }
                else
                {
                    xmlOut += "<item id=\"" + obj.ItemID.ToString("") + "\" modifieddate=\"" + obj.ModifiedDate.ToString("s") + "\">" + obj.XMLData + "</item>";
                }
            }
            return xmlOut;
        }

        public static string DoXslTransOnTemplate(string strTemplateText, NBrightInfo objInfo)
        {
            if (strTemplateText.ToLower().Contains("<xsl:stylesheet"))
            {
                var xmlOut = "<root>";
                var l = new List<NBrightInfo> { objInfo };
                xmlOut += NBrightStoreUtils.FormatListtoXml(l);
                xmlOut += "</root>";
                return XslUtils.XslTransInMemory(xmlOut, strTemplateText);
            }
            return strTemplateText;
        }

        public static string GetUniqueKeyRef(int PortalId, int ModuleId, string KeyRef, int LoopCount)
        {
            var rtnKeyRef = KeyRef;
            if (LoopCount > 0) rtnKeyRef += LoopCount.ToString("");

            var strFilter = " and ModuleId != " + ModuleId + " and GUIDKey = '" + rtnKeyRef + "' ";

            var l = CBO.FillCollection<NBrightInfo>(DataProvider.Instance().GetList(PortalId, -1, "SETTINGS", strFilter, "", 1, 1, 1, 1, ""));
            if (l.Count >= 1)
            {
                rtnKeyRef = GetUniqueKeyRef(PortalId, ModuleId, KeyRef, LoopCount + 1);
            }

            return rtnKeyRef;
        }

        public static void SetModCache(int moduleid, string CacheKey, object objObject)
        {
            var cList = (List<String>)NBrightCore.common.Utils.GetCache("keylist:" + moduleid.ToString(CultureInfo.InvariantCulture));
            if (cList == null) cList = new List<String>();
            if (!cList.Contains(CacheKey))
            {
                cList.Add(CacheKey);
                NBrightCore.common.Utils.SetCache("keylist:" + moduleid.ToString(CultureInfo.InvariantCulture), cList);
                NBrightCore.common.Utils.SetCache(CacheKey, objObject);
            }
        }

        public static void RemoveModCache(int moduleid)
        {
            var cList = (List<String>)NBrightCore.common.Utils.GetCache("keylist:" + moduleid.ToString(CultureInfo.InvariantCulture));
            if (cList != null)
            {
                foreach (var s in cList)
                {
                    NBrightCore.common.Utils.RemoveCache(s);
                }
            }
            NBrightCore.common.Utils.RemoveCache("keylist:" + moduleid.ToString(CultureInfo.InvariantCulture));
        }

        public static void RemoveModCachePortalWide(int portalid)
        {
            var mCtrl = new NBrightStoreController();
            var l = mCtrl.GetListInfo(portalid, -1, "SETTINGS");
            foreach (var obj in l)
            {
                RemoveModCache(obj.ModuleId);
            }

        }

        public static String GetResourceKeyPackageValue(int portalId, int moduleId, String ControlPath, String Lang, String Key)
        {
            var strValue = "";
            var resxList = GetResourcePackageData(portalId, moduleId, ControlPath, Lang);

            if (resxList.Count > 0)
            {
                if (resxList.ContainsKey(Key))
                {
                    strValue = resxList[Key];
                }
            }

            return strValue;
        }

        public static Dictionary<String, String> GetResourcePackageData(int portalId, int moduleId, String ControlPath, String Lang)
        {
            var rtnList = new Dictionary<String, String>();
            var NBSettings = GetSettings(portalId, moduleId);
            var resxList = GenXmlFunctions.GetCheckBoxListValues(NBSettings.XMLDoc, "genxml/checkboxlist/chkresxlist/chk");

            var resxLst = resxList.Split(',');
            foreach (var r in resxLst)
            {
                if (!string.IsNullOrEmpty(r))
                {
                    // load default into dictionary
                    var mapPath = System.Web.Hosting.HostingEnvironment.MapPath(ControlPath);
                    if (mapPath != null)
                    {
                        var fpath = mapPath.TrimEnd('\\') + "\\resx\\" + r + "." + Lang + ".resx";
                        if (System.IO.File.Exists(fpath))
                        {
                            rtnList = PopulateResourceDictionary(rtnList, fpath, r);
                        }
                        else
                        {
                            fpath = mapPath.TrimEnd('\\') + "\\resx\\" + r + ".resx";
                            if (System.IO.File.Exists(fpath))
                            {
                                rtnList = PopulateResourceDictionary(rtnList, fpath, r);
                            }

                        }
                    }

                    //save langauge dictionary to cache
                }
            }

            return rtnList;
        }

        private static Dictionary<String, String> PopulateResourceDictionary(Dictionary<String, String> rtnList, String ResourceFilePath, String ResourceFileName)
        {
            var xmlDoc = new XmlDataDocument();
            var strData = Utils.ReadFile(ResourceFilePath);
            if (strData != "")
            {
                try
                {
                    xmlDoc.LoadXml(strData);
                    var nodList = xmlDoc.SelectNodes("root/data");
                    if (nodList != null)
                    {
                        foreach (XmlNode nod in nodList)
                        {
                            if (nod.Attributes != null && nod.Attributes["name"] != null && nod.SelectSingleNode("./value") != null)
                            {
                                var key = ResourceFileName + "." + nod.Attributes["name"].InnerText;
                                var value = nod.SelectSingleNode("./value").InnerText;
                                if (!rtnList.ContainsKey(key))
                                {
                                    rtnList.Add(key, value);
                                }
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    //ignore error, might not be valid XML in the file
                }
            }
            return rtnList;
        }

        private const string PageBasedBackofficeFile = "/DesktopModules/NBright/NBrightStore/Manager.aspx";

        public static bool IsPageBasedBackOffice(HttpRequest httpRequest)
        {
            return httpRequest.Path.EndsWith(PageBasedBackofficeFile); 
        }

        public static string GetManagerPageBaseUrl()
        {
            return
                Globals.ResolveUrl(string.Format("~{0}", PageBasedBackofficeFile));
        }

        public static string GetManagerPageUrl(int backofficeModuleId)
        {
            return
                Globals.ResolveUrl(string.Format("~{0}?mid={1}", PageBasedBackofficeFile, backofficeModuleId));
        }

    }

}

