﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using DotNetNuke.Entities.Portals;
using DotNetNuke.Entities.Users;
using DotNetNuke.Modules.NBrightStore.Components;
using NBrightCore.common;
using NBrightCore.render;
using NBrightDNN;
using System.Xml;

namespace DotNetNuke.Modules.NBrightStore
{
    /// <summary>
    /// Summary description for XMLconnector
    /// </summary>
    public class XmlConnector : IHttpHandler
    {

        public void ProcessRequest(HttpContext context)
        {
            var strOut = "";

            if (UserController.GetCurrentUserInfo() != null)
            {

                var paramCmd = Utils.RequestQueryStringParam(context, "cmd");
                var itemId = Utils.RequestQueryStringParam(context, "itemid");
                var ctlType = Utils.RequestQueryStringParam(context, "ctltype");
                var idXref = Utils.RequestQueryStringParam(context, "idxref");
                var xpathpdf = Utils.RequestQueryStringParam(context, "pdf");
                var xpathref = Utils.RequestQueryStringParam(context, "pdfref");
                var lang = Utils.RequestQueryStringParam(context, "language");
                var moduleId = Utils.RequestQueryStringParam(context, "mid");
                var moduleKey = Utils.RequestQueryStringParam(context, "mkey");
				var parentid = Utils.RequestQueryStringParam(context, "parentid");
				var entryid = Utils.RequestQueryStringParam(context, "entryid");
				var entryxid = Utils.RequestQueryStringParam(context, "entryxid");
				var catid = Utils.RequestQueryStringParam(context, "catid");
				var catxid = Utils.RequestQueryStringParam(context, "catxid");
                var templatePrefix = Utils.RequestQueryStringParam(context, "tprefix");

				if (lang != null && lang != "" && System.Threading.Thread.CurrentThread.CurrentCulture.ToString() != lang)
				{
					// because we are using a webservice the system current thread culture might not be set correctly,
					//  so use the lanaguge param to set it.
 					// TODO: see if we can find a nicer way to do this.
					try
					{
						var c = new System.Globalization.CultureInfo(lang);
						System.Threading.Thread.CurrentThread.CurrentCulture = c;
					}
					catch
					{
						// invalid lang culture, so ignore and use the system default.
						lang = UserController.GetCurrentUserInfo().Profile.PreferredLocale;
					}
				}
                // default to current thread is we have no language.
                if (lang == "") lang = System.Threading.Thread.CurrentThread.CurrentCulture.ToString();

                var intModuleId = 0;
                if (Utils.IsNumeric(moduleId)) intModuleId = Convert.ToInt32(moduleId);

                var objCtrl = new NBrightStoreController();

				var uInfo = new UserDataInfo(UserController.GetCurrentUserInfo().PortalID, intModuleId, objCtrl, ctlType);

                NBrightInfo objPInfo;

                switch (paramCmd)
                {
                    case "test":
                        strOut = "<root>" + UserController.GetCurrentUserInfo().Username + "</root>";
                        break;
                    case "setdata":
                        break;
                    case "deldata":
                        break;
                    case "getdata":
                        break;
                    case "getcatxml":
						strOut = GetCatTreeXml(parentid);
                        break;
					case "setproductxref":
						#region "setentryxref code"

                        if (Utils.IsNumeric(entryid) && uInfo.FromItemId != entryid)
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
								var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    obj.AddXref("prdxref", uInfo.FromItemId);
                                    objPInfo.AddXref("prdxref", entryid);

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetRelatedProducts(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
						#endregion
                        break;
                    case "delproductxref":
						#region "delproductxref code"
                        if (Utils.IsNumeric(entryid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
								var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    //Removexref node, if there.
                                    obj.RemoveXref("prdxref", uInfo.FromItemId);
                                    objPInfo.RemoveXref("prdxref", entryid);

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetRelatedProducts(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
						#endregion
						break;
                    case "getproductxref":
                        strOut = ProductFunctions.GetRelatedProducts(UserController.GetCurrentUserInfo().PortalID, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                        break;
                    case "setproductimg":
                        #region "setentryimg code"

                        if (Utils.IsNumeric(entryid) && uInfo.FromItemId != entryid)
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
                                var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    obj.AddXref("prdimg", uInfo.FromItemId);
                                    objPInfo.AddXref("prdimg", entryid);

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetProductImgs(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
                        #endregion
                        break;
                    case "delproductimg":
                        #region "delproductimg code"
                        if (Utils.IsNumeric(entryid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
                                var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    //Removexref node, if there.
                                    obj.RemoveXref("prdimg", uInfo.FromItemId);
                                    objPInfo.RemoveXref("prdimg", entryid);

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetProductImgs(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
                        #endregion
                        break;
                    case "getproductimg":
                        strOut = ProductFunctions.GetProductImgs(UserController.GetCurrentUserInfo().PortalID, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                        break;
                    case "setproductopt":
                        #region "setentryopt code"

                        if (Utils.IsNumeric(entryid) && uInfo.FromItemId != entryid)
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
                                var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    obj.AddXref("prdopt", uInfo.FromItemId);
                                    objPInfo.AddXref("prdopt", entryid);

                                    //calculate models 
                                    objPInfo = ProductFunctions.CalculateModels(objPInfo, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetProductOpts(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
                        #endregion
                        break;
                    case "delproductopt":
                        #region "delproductopt code"
                        if (Utils.IsNumeric(entryid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
                                var obj = objCtrl.GetInfo(Convert.ToInt32(entryid));
                                if (obj != null && Utils.IsNumeric(uInfo.FromItemId))
                                {
                                    objPInfo = objCtrl.GetInfo(Convert.ToInt32(uInfo.FromItemId));

                                    //Removexref node, if there.
                                    obj.RemoveXref("prdopt", uInfo.FromItemId);
                                    objPInfo.RemoveXref("prdopt", entryid);

                                    //calculate models 
                                    objPInfo = ProductFunctions.CalculateModels(objPInfo, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));

                                    //commit to DB
                                    objCtrl.UpdateInfo(obj);
                                    objCtrl.UpdateInfo(objPInfo);

                                    // return updated list
                                    strOut = ProductFunctions.GetProductOpts(obj.PortalId, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
                        #endregion
                        break;
                    case "getproductopt":
                        strOut = ProductFunctions.GetProductOpts(UserController.GetCurrentUserInfo().PortalID, uInfo.FromItemId, lang, templatePrefix, HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                        break;
                    case "setcatxref":
						#region "setcatref code"
						if (Utils.IsNumeric(catid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
								var obj = objCtrl.GetInfo(Convert.ToInt32(catid));
								var eXRef = objCtrl.GetInfoByGuidKey(obj.PortalId, obj.ModuleId, "CATXREF", obj.ItemID.ToString("") + "x" + uInfo.FromItemId);
								if (eXRef == null && Utils.IsNumeric(uInfo.FromItemId))
                                    {
										if (obj.ItemID.ToString("") != uInfo.FromItemId)
                                        {
                                            eXRef = new NBrightInfo();
                                            eXRef.ItemID = -1;
											eXRef.GUIDKey = obj.ItemID.ToString("") + "x" + uInfo.FromItemId;
                                            eXRef.TypeCode = "CATXREF";
                                            eXRef.ModuleId = -1;
                                            eXRef.PortalId = obj.PortalId;
											eXRef.XMLData = "<genxml><hidden/></genxml>"; // set to empty root, so we can add the default cat flag.
											eXRef.ParentItemId = Convert.ToInt32(uInfo.FromItemId);
                                            eXRef.XrefItemId = obj.ItemID;
                                            objCtrl.UpdateInfo(eXRef);
                                        }
                                    }
                                strOut = ProductFunctions.GetRelatedCats(obj.PortalId, uInfo.FromItemId, lang, "AdminCatXref", HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"), false);
                            }
                        }
						#endregion
                        break;
                    case "delcatxref":
						#region "delcatxref code"
						if (Utils.IsNumeric(catxid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
								var obj = objCtrl.GetInfo(Convert.ToInt32(catxid));
                                if (obj != null)
                                {
                                    if (obj.TypeCode == "CATXREF") //make sure we are deleting correct type
                                    {
										objCtrl.DeleteInfo(Convert.ToInt32(catxid));
                                    }
                                    strOut = ProductFunctions.GetRelatedCats(obj.PortalId, obj.ParentItemId.ToString(""), lang, "AdminCatXref", HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"), false);
                                }
                            }
                        }
						#endregion
                        break;
                    case "getcatxref":
                        strOut = ProductFunctions.GetRelatedCats(UserController.GetCurrentUserInfo().PortalID, uInfo.FromItemId, lang, "AdminCatXref", HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
						break;
                    case "catxrefdefault":
						#region "catxrefdefault code"
						if (Utils.IsNumeric(catxid))
                        {
                            if (UserController.GetCurrentUserInfo().IsInRole("Manager") | UserController.GetCurrentUserInfo().IsInRole("manager"))
                            {
								var obj = objCtrl.GetInfo(Convert.ToInt32(catxid));
                                if (obj != null)
                                {
                                    var objInfo = objCtrl.GetInfo(obj.ParentItemId);
                                        if (objInfo != null)
                                        {
                                            // clear all existing defaults
                                            var strFilter = " and parentitemid = " + objInfo.ItemID.ToString("") + " ";
                                            var l = objCtrl.GetListInfo(obj.PortalId, obj.ModuleId, "CATXREF", strFilter);
                                            foreach (var o in l)
                                            {
                                                o.SetXmlProperty("genxml/hidden/defaultcat", "");
                                                objCtrl.UpdateInfo(o);
                                            }
                                            // set in the parent itemid
                                            objInfo.SetXmlProperty("genxml/hidden/defaultcatid", obj.XrefItemId.ToString(""));
                                            objCtrl.UpdateInfo(objInfo);
                                            // set in the xref ,for qucik display
                                            obj.SetXmlProperty("genxml/hidden/defaultcat", "True");
                                            objCtrl.UpdateInfo(obj);
                                        }
                                        strOut = ProductFunctions.GetRelatedCats(obj.PortalId, obj.ParentItemId.ToString(), lang, "AdminCatXref", HttpContext.Current.Server.MapPath("/DesktopModules/NBright/NBrightStore"));
                                }
                            }
                        }
						#endregion
						break;

                }
            }



            //send back xml as plain text
            context.Response.Clear();
            context.Response.ContentType = "text/plain";
            if (strOut == "")
            {
                context.Response.Write("404 Not Found");
                context.Response.StatusCode = 404;
            }
            else
            {
                context.Response.Write(strOut);
            }
            context.Response.End();
        }



        private string GetReturnData(HttpContext context, string ctlType, string moduleId)
        {
            if (!Utils.IsNumeric(moduleId))
            {
                return "ERROR: No ModuleId passed to the Ajax function.";
            }

            var objCtrl = new NBrightStoreController();

            var queryStrings = "";
            int loop1, loop2;
            // Load NameValueCollection object.
            NameValueCollection coll = context.Request.QueryString;
            // Get names of all keys into a string array.
            String[] arr1 = coll.AllKeys;
            for (loop1 = 0; loop1 < arr1.Length; loop1++)
            {
                if (arr1[loop1].Length > 1) // remove postback, unrequired params
                {
                    queryStrings += " " + arr1[loop1] + "=\"";
                    String[] arr2 = coll.GetValues(arr1[loop1]);
                    if (arr2 != null)
                    {
                        for (loop2 = 0; loop2 < arr2.Length; loop2++)
                        {
                            queryStrings += arr2[loop2] + "\" ";
                        }                        
                    }
                }
            }

            var strOut = "";
            var sqlSearchFilter = Utils.RequestQueryStringParam(context, "filter");
            var sqlOrderBy = Utils.RequestQueryStringParam(context, "orderby");

            var uInfo = new UserDataInfo(PortalSettings.Current.PortalId, Convert.ToInt32(moduleId), objCtrl, "NBrightAjax");

            var currentTabID =  uInfo.TabId;
            var currentModID = uInfo.ModuleId;
            var skinSrc = uInfo.SkinSrc;
            var objList = objCtrl.GetListInfo(PortalSettings.Current.PortalId, currentModID, ctlType, sqlSearchFilter, sqlOrderBy);
            var xmlOut = "<root tabid=\"" + currentTabID + "\" mid=\"" + currentModID + "\" skinsrc=\"" + skinSrc + "\" " + queryStrings + ">";
            foreach (var obj in objList)
            {
                if (obj.XMLData.StartsWith("<root>"))
                {
                    // Data is Xref, so read actual data required.
                    var xrefId = GenXmlFunctions.GetGenXmlValue(obj.XMLData, "root/xref");
                    if (Utils.IsNumeric(xrefId))
                    {
                        var obj2 = objCtrl.GetInfo(Convert.ToInt32(xrefId));
                        if (obj2 != null)
                        {
                            var linedxrefId = GenXmlFunctions.GetGenXmlValue(obj.XMLData, "root/linkedxref");
                            xmlOut += "<item id=\"" + obj.ItemID.ToString("") + "\" idxref=\"" + linedxrefId + "\">" + obj2.XMLData + "</item>";
                        }
                    }
                }
                else
                {
                    xmlOut += "<item id=\"" + obj.ItemID.ToString("") + "\">" + obj.XMLData + "</item>";
                }
            }
            xmlOut += "</root>";

            var templ = Utils.RequestQueryStringParam(context, "templ");
            var MInfo = NBrightDNN.DnnUtils.GetModuleinfo(currentModID);
            if (MInfo != null)
            {
                // TODO: needs to be controlMapPath NOT ContainerPath.
                var objTCtrl = new NBrightCore.TemplateEngine.TemplateController(MInfo.ContainerPath);
                var xslData = objTCtrl.GetTemplateData(templ, Utils.GetCurrentCulture());
                try
                {
                    strOut = NBrightCore.common.XslUtils.XslTransInMemory(xmlOut, xslData);
                }
                catch (Exception)
                {
                    strOut = "ERROR: " + xslData;
                }
            }
            else
            {
                strOut = "ERROR: No ModuleInfo";
            }
            return strOut;
        }

		private string GetCatTreeXml ( string parentItemId = "0", bool recursive = true, int depth = 0 )
		{

			if (!Utils.IsNumeric(parentItemId)) parentItemId = "0";
			var strOut = "";
			var objCtrl = new NBrightStoreController();
			var strFilter = " and ParentItemId = " + parentItemId + " ";
			var strOrderBy = " order by [XMLData].value('(genxml/hidden/sortorder)[1]','nvarchar(10)'),[XMLData].value('(genxml/textbox/txtname)[1]','nvarchar(50)') ";

			if (parentItemId == "0" | recursive == false)
			{
				strOut = "<root>";
			}

			var l = objCtrl.GetListInfo(PortalSettings.Current.PortalId, -1, "CATEGORY", strFilter, strOrderBy);

			var lp = 0;
			foreach (var obj in l)
			{
				lp = lp + 1;
				strOut += "<node";
				strOut += " id='" + obj.ItemID.ToString("") + "' ";
				strOut += ">";
				strOut += obj.XMLData;
				if (recursive && depth < 10)
				{
					depth = depth + 1;
					strOut += GetCatTreeXml(obj.ItemID.ToString(""), true, depth);
				}
				strOut += "</node>";
			}

			if (parentItemId == "0")
			{
				strOut += "</root>";
			}
			return strOut;
		}

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }
    }
}